package NetHirc::Channel;

use strict;

use Carp;

use constant NO_CHANNEL => "no channel";

my @channels = ();
my %old = ();

sub new {
    my $proto = shift;
    my $class = ref $proto || $proto;
    my $channel = shift;
    my $conn = shift;
    unless ($class->is_channel_name($channel)) {
	croak __PACKAGE__, "::new -- bad channel name";
    }
    unless (ref($conn) eq 'Net::IRC::Connection') {
	croak __PACKAGE__, "::new -- no connection?!";
    }
    my $self = [ $channel, $conn ];
    return bless $self, $class;
}

# Static class method
sub is_channel_name {
    shift;
    my $channel = shift;
    return ($channel =~ /^[#&!+]/);
}

# Static class method
sub add {
    my $c = new(@_);
    push @channels, $c;
    @channels = grep { !(($_->[1] eq $c->[1]) && ($_-[0] eq NO_CHANNEL)) } @channels;
}

# Static class method
sub current { shift; $channels[0] }

sub name { $_[0]->[0] }
sub conn { $_[0]->[1] }
sub server { $_[0]->[1]->server() }

# Static class method
sub set { 
    shift;
    my $channel = shift;
    my $conn = shift;
    my $item;
    my @net = grep { $_->[0] eq $channel } @channels;
    if (not @net) {
	return undef;
    }
    if ((@net > 1) && $conn) {
	@net = grep { $_->[1] eq $conn } @net;
    }
    $item = $net[0];
    rotate_to_front($item);
    return $item;
}

sub remove { 
    shift;
    my $channel = shift;
    my $conn = shift;
    @channels = grep { ($_->[1] ne $conn) || ($_->[0] ne $channel) } @channels;
}

sub rotate_to_front {
    my $item = shift;
    while ($channels[0] ne $item) {
	push @channels, shift @channels;
    }
}

sub list { shift; grep { $_->[0] } @channels }

sub disconnect {
    shift;
    my $conn = shift;
    @channels = grep { $_->[1] eq $conn and $_->[0] ne NO_CHANNEL } @channels;
    my @lost = grep { $_->[1] eq $conn } @channels;
    map { remove(__PACKAGE__, @$_) } @lost;
    $old{$conn->server()} = \@lost;
}

sub needs_reconnect {
    shift;
    my $conn = shift;
    my $server = $conn->server();
    return unless exists $old{$server};
    my @found = @{$old{$conn->server()}};
    delete $old{$conn->server()};
    return @found;
}

sub next {
    while ($channels[1]->[0] eq NO_CHANNEL) { 
	splice @channels, 1, 1;
    }
    set($_[0], $channels[1]->[0], $channels[1]->[1]);
    current($_[0]);
}

sub nochannel {
    shift;
    my $conn = shift;
    my $channel = bless [ NO_CHANNEL, $conn ], __PACKAGE__;
    unshift @channels, $channel;
}

1;
