# (c) 2002 Tony Monroe
package NetHirc::Formats;

use strict;

use vars qw( 
@ISA 
@EXPORT 
%irc_formats 
@deities 
@classes 
@ordinary_monsters 
@unique_monsters 
@scrolls 
%self_mode_comments 
%channel_mode_comments 
@disasters 
@weapons 
);

use Exporter;
use Term::Cap;

@ISA = qw(Exporter);

@EXPORT = qw( 
%irc_formats 
@deities 
@classes 
@ordinary_monsters 
@unique_monsters 
@scrolls 
%self_mode_comments 
%channel_mode_comments 
@disasters 
&ircformat 
@weapons 
);

my $boldon = "\cb";
my $boldoff = "\cb";
my $invon = "\cv";
my $invoff = "\cv";
my $underon = "\c_";
my $underoff = "\c_";

%irc_formats = ();

sub ircformat {
    my $format_name = shift;
    return sprintf $irc_formats{$format_name}, @_;
}

sub load_formats {
    my $line;
    while ($line = <DATA>) {
	my ($k, $v) = split(' ', $line, 2);
	$irc_formats{$k} = $v;
    }
}

sub random_monster {
    my $ordinaries = scalar @ordinary_monsters;
    my $n = int(rand(@ordinary_monsters + @unique_monsters));
    if ($n > $#ordinary_monsters) {
	$n -= $ordinaries;
	return $unique_monsters[$n];
    }
    return "the " . $ordinary_monsters[$n];
}

load_formats() unless $irc_formats{'Xversion'};

@deities = qw( 
Anhur 
Thoth 
Ptah 
Loki 
Odin 
Offler 
Susanowo 
Raijin 
Issek 
Brigit 
Lugh 
Poseidon 
Hermes
Athena
Anshar
Ishtar
Crom
Mitra
Huhetotl
Camaxtli
Quetzalcoatl
Moloch
Erevan_Ilesere
Aerdrie_Faenya
Solonor_Thelandira
The_Lady
Blind_Io
Amaterasu_Omikami
Huan_Ti
Chih_Sung-tzu
Shan_Lai_Ching
Manannan_Mac_Lir
);

@classes = qw(
Archaeologist
Barbarian
Caveman
Elf
Healer
Knight
Priest
Rogue
Samurai
Tourist
Valkyrie
Wizard
);


@ordinary_monsters = qw(
giant_ant
killer_bee
soldier_ant
fire_ant
giant_beetle
queen_bee
acid_blob
quivering_blob
gelatinous_cube
cockatrice
jackal
werejackal
little_dog
dog
large_dog
wolf
werewolf
warg
winter_wolf
hell_hound_pup
hell_hound
Cerberus
floating_eye
freezing_sphere
beholder
kitten
housecat
jaguar
large_cat
tiger
gremlin
gargoyle
winged_gargoyle
hobbit
dwarf
bugbear
dwarf_lord
dwarf_king
mind_flayer
manes
homunculus
imp
lemure
quasit
tengu
blue_jelly
spotted_jelly
ochre_jelly
kobold
large_kobold
kobold_lord
kobold_shaman
leprechaun
small_mimic
large_mimic
giant_mimic
wood_nymph
water_nymph
mountain_nymph
goblin
hobgoblin
orc
hill_orc
Mordor_orc
Uruk-hai
orc_shaman
orc-captain
rock_piercer
iron_piercer
rothe
mumak
leocrotta
wumpus
titanothere
baluchitherium
sewer_rat
giant_rat
rabid_rat
wererat
rock_mole
cave_spider
giant_spider
scorpion
lurker_above
trapper
white_unicorn
gray_unicorn
black_unicorn
fog_cloud
dust_vortex
ice_vortex
energy_vortex
steam_vortex
fire_vortex
baby_long_worm
baby_purple_worm
long_worm
purple_worm
grid_bug
xan
yellow_light
black_light
zruty
couatl
Aleax
Angel
ki-rin
Archon
bat
giant_bat
vampire_bat
plains_centaur
forest_centaur
mountain_centaur
baby_gray_dragon
baby_red_dragon
baby_white_dragon
baby_orange_dragon
baby_black_dragon
baby_blue_dragon
baby_green_dragon
baby_yellow_dragon
gray_dragon
red_dragon
white_dragon
orange_dragon
black_dragon
blue_dragon
green_dragon
yellow_dragon
air_elemental
fire_elemental
earth_elemental
water_elemental
brown_mold
yellow_mold
green_mold
red_mold
shrieker
violet_fungus
gnome
gnome_lord
gnomish_wizard
gnome_king
giant
stone_giant
hill_giant
fire_giant
frost_giant
ettin
titan
minotaur
stalker
jabberwock
Keystone_Kop
Kop_Sergeant
Kop_Lieutenant
Kop_Kaptain
lich
demilich
master_lich
kobold_mummy
gnome_mummy
orc_mummy
elf_mummy
human_mummy
ettin_mummy
giant_mummy
red_naga_hatchling
black_naga_hatchling
golden_naga_hatchling
guardian_naga_hatchling
red_naga
black_naga
golden_naga
guardian_naga
ogre
ogre_lord
ogre_king
gray_ooze
brown_pudding
black_pudding
quantum_mechanic
rust_monster
garter_snake
snake
water_moccasin
pit_viper
python
cobra
troll
ice_troll
rock_troll
water_troll
Olog-hai
umber_hulk
vampire
vampire_lord
Vlad_the_Impaler
barrow_wight
wraith
Nazgul
xorn
ape
owlbear
yeti
carnivorous_ape
sasquatch
kobold_zombie
gnome_zombie
orc_zombie
elf_zombie
human_zombie
ettin_zombie
giant_zombie
skeleton
straw_golem
rope_golem
leather_golem
wood_golem
flesh_golem
clay_golem
stone_golem
iron_golem
human
wererat
werejackal
werewolf
Woodland-elf
Green-elf
Grey-elf
elf-lord
Elvenking
nurse
shopkeeper
guard
Oracle
aligned_priest
high_priest
soldier
sergeant
lieutenant
captain
watchman
watch_captain
Wizard_of_Yendor
ghost
shade
water_demon
horned_devil
succubus
incubus
erinys
barbed_devil
marilith
vrock
hezrou
bone_devil
ice_devil
nalfeshnee
pit_fiend
balrog
mail_daemon
djinni
giant_eel
electric_eel
kraken
newt
gecko
iguana
baby_crocodile
lizard
chameleon
crocodile
salamander
long_worm_tail
archeologist
barbarian
caveman
cavewoman
elf
healer
knight
priest
priestess
rogue
samurai
tourist
valkyrie
wizard
Elwing
Arch_Priest
Master_of_Thieves
Twoflower
Norn
Wizard_of_Balance
Minion_of_Huhetotl
Chromatic_Dragon
Goblin_King
Cyclops
Ixoth
Nalzok
Master_Assassin
Dark_One
student
chieftain
neanderthal
High-elf
attendant
page
acolyte
thug
ninja
roshi
guide
warrior
apprentice
jumbo_shrimp
giant_pigmy
gnu
killer_penguin
giant_cockroach
giant_slug
maggot
pterodactyl
tyrannosaurus_rex
basilisk
beholder
nightmare
efreeti
marid
rot_grub
bookworm
doppelganger
shadow
hologram
jester
attorney
sleazoid
killer_tomato
amazon
robot
battlemech
rhinovirus
harpy
lion-dog
rat-ant
grue
Christmas-tree_monster
luck_sucker
paskald
brogmoid
dornbeast
Ancient_Multi-Hued_Dragon
emu
kestrel
xeroc
venus_flytrap
creeping_coins
hydra
siren
killer_bunny
rodent_of_unusual_size
Luggage
Ent
tangle_tree
nickelpede
wiggle
white_rabbit
snark
pushmi-pullyu
smurf
tribble
Klingon
Borg
Ewok
Totoro
ohmu
youma
earthquake_beast
Invid
Terminator
boomer
Dalek
microscopic_space_fleet
Ravenous_Bugblatter_Beast_of_Traal
teenage_mutant_ninja_turtle
samurai_rabbit
aardvark
witch_doctor
one-eyed_one-horned_flying_purple_people_eater
fiendish_vampiric_half-dragon_were-beholder
were-shriner
Justice_Bringers
Fascist_Four
);

@unique_monsters = qw(
Barney_the_dinosaur
Medusa
Croesus
Charon
Juiblex
Yeenoghu
Orcus
Geryon
Dispater
Baalzebub
Asmodeus
Demogorgon
Death
Pestilence
Famine
Lord_Carnarvon
Pelias
Shaman_Karnov
Earendil
Hippocrates
King_Arthur
Lord_Sato
Ashikaga_Takauji
Lord_Surtur
Thoth_Amon
Evil_Iggy
Smokey_the_bear
Godzilla
King_Kong
Audrey_II
Mighty_Joe_Young
Mighty_Joe_Jung
Phil,_the_Prince_of_Insufficent_Light
Commander_Skullpuss
);

@scrolls = qw(
KIRJE
READ_ME
GARVEN_DEH
TEMOV
FOOBIE_BLETCH
VELOX_NEB
HACKEM_MUCHE
VE_FORBRYDERNE
ANDOVA_BEGARIN
DUAM_XNAHT
ELAM_EBOW
KERNOD_WEL
YUM_YUM
THARR
VENZAR_BORGAVVE
VERR_YED HORRE
ELBIB_YLOH
PRIRUTSENIE
LEP_GEX VEN ZEA
DAIYEN_FOOELS
PRATYAVAYAH
XIXAXA_XOXAXA XUXAXA
JUYED_AWK YACC
ZELGO_MER
);

@disasters = qw(
You_hear_a_distant_squeak.
A_board_squeaks_loudly_under_your_feet.
You_hear_a_distant_explosion.
KABLAMM!!__You_triggered_a_land_mine!
You_are_caught_in_a_bear_trap!
You_fall_down_the_stairs.
You_are_frozen_by_the_floating_eye's_gaze!
You_are_splashed_by_acid!
You_feel_guilty.
You_have_a_sad_feeling_for_a_moment,_then_it_passes.
Your_brain_is_on_fire!
You_choke_on_your_food.
You_hear_the_footsteps_of_a_guard_on_patrol.
You_hear_the_chime_of_a_cash_register.
The_air_crackles_around_the_gnomish_wizard.
You_rebalance_your_load.__Movement_is_difficult.
You_hear_a_monster_behind_the_boulder._
KLUNK!__The_pipes_vibrate_noisily.
You_summoned_a_water_demon!
Oh,_wow,_great_stuff!__Everything_looks_so_cosmic_now!
You_stagger...
You_hear_an_elephant_stepping_on_a_peanut.
Whew!__You_survived_that_attempt_on_your_life.
This_tepid_water_is_tasteless.
You_faint_from_lack_of_food.
There_was_a_rattlesnake_hiding_under_the_statue_of_a_mountain_centaur!
Wait,_that's_a_large_mimic!
The_leprechaun_hits!__Your_purse_feels_lighter.
);

@weapons = qw(
a_long_sword_named_Excalibur
a_runesword_named_Stormbringer
a_war_hammer_named_Mjollnir
a_battle_axe_named_Cleaver
an_Orcish_dagger_named_Grimtooth
an_Elven_broadsword_named_Orcrist
an_Elven_dagger_named_Sting
an_athame_named_Magicbane
a_long_sword_named_Frost_Brand
a_long_sword_named_Fire_Brand
a_broadsword_named_Dragonbane
a_long_sword_named_Demonbane
a_silver_saber_named_Werebane
a_silver_saber_named_Grayswandir
a_long_sword_named_Giantslayer
a_war_hammer_named_Ogresmasher
a_morning_star_named_Trollsbane
a_long_sword_named_Vorpal_Blade
a_katana_named_Snickersnee
a_long_sword_named_Sunsword
the_Sceptre_of_Might
the_Staff_of_Aesculapius
the_Longbow_of_Diana
the_Tsurugi_of_Muramasa
);

map { s/_/ /g } @ordinary_monsters, @unique_monsters, @deities, @scrolls, @disasters, @weapons;

%self_mode_comments = (
'+c' =>	"^^^^ You are involuntarily tuned into the security camera.",
'-c' =>	"^^^^ Your no longer see images from the security camera.",
'+d' =>	"^^^^ The inner workings of the server become transparent to you.",
'-d' =>	"^^^^ The inner workings of the server become opaque to you.",
'+f' =>	"^^^^ Your I-line sensors are magically repaired!",
'-f' =>	"^^^^ Your I-line sensors are hopelessly scrambled.",
'+i' =>	"^^^^ You can barely see yourself.",
'-i' =>	"^^^^ You appear solid again.",
'+k' =>	"^^^^ The server's slavering fangs become apparent to you.",
'-k' =>	"^^^^ The server's slavering fangs are no longer visible.",
'+n' =>	"^^^^ You feel like you can detect transformations...",
'-n' =>	"^^^^ You no longer feel like you can detect transformations.",
'+o' =>	"^^^^ You glow silver for a moment.",
'-o' =>	"^^^^ You glow black for a moment.",
'+r' =>	"^^^^ You are given a radio link to the Rejection Department.",
'-r' =>	"^^^^ Your radio link to the Rejection Department is taken away!",
'+s' =>	"^^^^ Your hear faint mumblings in the distance.",
'-s' =>	"^^^^ You no longer hear faint mumblings in the distance.",
'+w' =>	"^^^^ You see words forming in the clouds in the sky...",
'-w' =>	"^^^^ The cloud shapes no longer make sense to you.",
'+y' =>	"^^^^ You hear loud mumblings in a nearby room.",
'-y' =>	"^^^^ The loud mumblings seem to have ceased suddenly.",
'+z' =>	"^^^^ You feel mentally linked to other artifact-wielders.",
'-z' =>	"^^^^ You no longer feel mentally linked to other artifact-wielders.",
);

%channel_mode_comments = (
'+b' =>	"^^^^ In %s, a sense of foreboding surrounds %s...",
'-b' =>	"^^^^ In %s, the ward against %s is lifted!",
'+i' =>	"^^^^ A sign appears outside %s: \"By Invitation Only!\"",
'-i' =>	"^^^^ The \"By Invitation Only!\" sign outside %s crumbles to dust!",
'+k' =>	"^^^^ You hear someone locking the %s door...from the outside.",
'-k' =>	"^^^^ You hear someone unlocking the %s door.",
'+l' =>	"^^^^ %s suddenly feels a lot more cramped, like it could only hold %d creatures.",
'-l' =>	"^^^^ %s suddenly feels a lot less cramped.",
'+m' =>	"^^^^ A strange silence falls over %s.",
'-m' =>	"^^^^ Suddenly, it is noisier in %s.",
'+n' =>	"^^^^ The windows of %s glow more brightly now.",
'-n' =>	"^^^^ The windows of %s are not glowing as brightly now.",
'+o' =>	"^^^^ In %s, %s glows silver for a moment.",
'-o' =>	"^^^^ In %s, %s glows black for a moment.",
'+p' =>	"^^^^ A sign appears on the outer doorknob of %s: \"Do Not Disturb!\"",
'-p' =>	"^^^^ The \"Do Not Disturb\" sign is removed from the outer doorknob of %s.",
'+s' =>	"^^^^ Suddenly, you can't see the door to %s!",
'-s' =>	"^^^^ The door to %s reappears!",
'+t' =>	"^^^^ The signpost outside %s is surrounded by a shimmering light!",
'-t' =>	"^^^^ The shimmering light around the signpost outside %s fades.",
'+v' =>	"^^^^ In %s, the smelly sock vanishes from %s's mouth!",
'-v' =>	"^^^^ In %s, a smelly sock appears in %s's mouth.",
);

__DATA__
Xversion	$Id: Formats.pm,v 1.12 2004/05/04 20:55:45 tony Exp $
Caccepted	## Excellent, then we shall proceed.
Clist		?? Do you *really* want this Big Long List? [yn]
Cnames		?? Do you *really* want this Big Long List of Names? [yn]
Crefused	## Fine, see if I care!
Ebadchannel	## You think to yourself, "What a funny name for a channel!"
Ecantlog	## A logging daemon appears, hands you a card that reads "%s," and vanishes in a huff.
Eecho		## %s %s %s...
Eeval		## The voice of The Dark One booms: %s
Eignore		## You think to yourself, "What a strange thing to ignore!"
Eison		## Being confused, you pray to the wrong god!
Elogging	## You already have a logging daemon following you!
Eme		## You flap your arms in vain.  It is cold and empty out here.
Emsg		## You think to yourself, "What a strange name for a creature!"
Enocmd		## You don't know any spells named "%s".
Enocmdsrc	## You don't know the "%s" spell mentioned on line %d of the "%s" scroll.
Enotinchannel	## You don't have %s in your knapsack!
Enotlogging	## You can't see any logging daemon here.
Epart		## You may not part with what you do not specify!
Epipe		## KABLAMM!  Your piped process triggered a land mine: %s
Epipesay	## Your pipe falls to the ground and clatters loudly.
Esay		## You shout into the darkness.  The silence out here is oppressively heavy.
Esconnect	## The key fits in the lock, but does not turn.
Eserver		## The Great Wizards are too busy ignoring you.
Esource		## The succubus whispers gently in your ear: "%s, lover."
Esquit0		## You start bashing %s with your scroll labeled %s.
Esquit1		## %s does not appear amused.
Estats		## You think to yourself, "What a strange statistic!"
Esummon		## It's difficult to summon infernal creatures without knowing their names.
Esystem0	## You hear a distant explosion.
Esystem1	## There is something written here in the dust: "Exit code %d"
Etweak		## The spellbook's runes say: "%s"
Euserhost	## You feel a strange sense of emptiness.
Ewallops	## This scroll is blank.
Ewhois		## You feel materially poor.
Ewhowas		## Your memory seems clouded.
Poper		$$ So you wanna be an IRCop, eh?  What's the secret handshake?
Whappyslash	## Happyslash! You meant to type that, right?
Woperpasswd	## I hope that's your password, 'cuz I'm sending it...
Wpart		## I'll assume that you want to leave %s.
action		(*) %s %s
admin		## %s
adminemail	|==| "The administrator may be contacted at %s."
adminloc	|==| %s
adminme		|==| A scroll labeled "%s" appears in your hand.
away		{{ %s is away: %s
banlist		[| In %s, the ban against %s was placed by %s on %s.
bigbrotherl	**** Big Brother will remember up to %d words.
bigbrother_off0	**** Greetings, Citizen!  You have disabled Big Brother.
bigbrother_off1	**** You are making a mistake.  Big Brother is your friend!
bigbrother_on0	**** Greetings, Citizen!  You have enabled Big Brother.
bigbrother_on1	**** You have made the right decision.  Big Brother is your friend!
bigbrotherp	**** Probabilities are %d to revise, %d to keep, %d to discard, %d to announce.
bigbrotherstat	**** Big Brother is %s.  Have a nice day.
bigbrothersyn	**** Your words offend Big Brother.  Have a nice day.
cannotsendtochan	## Your spell bounces harmlessly off of %s.
channelcreate	## From your careful analysis, you can tell that %s has been here since %s.
channelmode	^^^^ In %s, %s reads a scroll of %s%s.
channelmodeis	## The door to %s has the ancient runes %s inscribed upon it.
channels	## Your current inventory: %s
chanswitch	## You spin and face %s.
clientinfo	Really read a scroll labelled %s? [ynq]
complaint	@*?!$%% %s
connect0	-- Hello, %s, Welcome to NetHirc!
connect1	-- You have been commanded to retrieve the Amulet of Yendor.
connect2	-- Go with the wisdom of %s, young %s!
count		## %-12s %7d lines %7d words %7d bytes
countclose	## You drop the counter named %s.
countfooter	## The Great Wizard %s finishes telling you of numbers.
countheader	## The Great Wizard %s gives you some numbers:
countopen	## You pick up a counter named %s.
countstate	## Your counter called "%s" is currently %s.
csource		still under development; ask me privately
ctcp		[CTCP] %s
debugstatus	~*~*~*~*~ Debugging is now %s ~*~*~*~*~
disconnect	*POOF!* You are cut off from %s!  You hear "%s" in the distance.
echo_off	## The acoustics of the room change subtly.
echo_on		## Echo echo echo...
error		$! %s
finger		%s the %s gets angry!
goof		## The voice of %s booms: "%s, mortal!"
goof0		## The voice of %s booms: "%s"
hsstatus	## Happyslash handling is currently %s.
invite		(( %s beckons seductively to you, "Come join me in %s."
inviting	)) You beckon seductively to %s, "Come join me in %s."
isnton		_\__ Your wand detects nothing.
ison		__/_ Your wand detects something: %s
join		>> %s (%s) has wandered into %s.
joining		## You see a door labeled %s here.  You move to open it...
kick		*OUCH!* %s yells "%s" and kicks %s out of %s!
kill		*OUCH!* %s hits you with %s!  You have died.
lastchannel	## You have been kicked out of your last channel!
links		-=-=- %s %s %s
list		|--| In %s, there are %d creatures discussing %s.
list1		|--| In %s, there is %d creature discussing %s.
listend		|==| The Great Wizard %s concludes the Ritual of Lists.
liststart	|==| The Great Wizard %s begins the Ritual of Lists.
logfilename	nethirc.%d.%d.log
logstart0	## A logging daemon appears!
logstart1	## The logging daemon scribbles on a scroll labeled '%s'.
logstarttext	********** nethirc log pid %d started %s UTC **********
logstop0	## The logging daemon vanishes! 
logstop1	## The scroll labeled '%s' drops to the floor.
logstoptext	********** nethirc log pid %s stopped %s UTC **********
motd		%s
mynick		## You shudder and change from %s into %s!
namelist	===> 	%s
names		===> You see %d creatures in %s.
needmoreparams	## You feel that your %s spell is missing something.
nextchannel	## You turn and face %s.
nextchannel0	## You turn and face the %s of the Great Wizard %s.
nick		## %s shudders and changes into %s!
nicknameinuse	!! That nick is taken, please choose another one now.
nochanlist	## Your knapsack is empty!
nochannel	## You don't have %s in your knapsack.
nochannels	## It is pitch black.  You might get eaten by a grue.
nocount		## Your knapsack doesn't have any counters in it.
nomotd		## The Great Wizard %s currently has no great wisdom to impart to you.
nosuchchannel	## You can't see any %s here.
nosuchnick	## You can't see anyone named %s here.
nosuchserver	## You look for a Great Wizard named %s, but you fail to find one.
notice		-%s- %s
onotice		-%s/%s- %s
notopic		## Nothing is written on the floor of %s.
oaction		(*) %s/%s %s
ooaction	(*) %s/%s/%s %s
ochan		<%s/%s> %s
onechannel	## Your knapsack only has one item in it: %s
oochan		<%s/%s/%s> %s
part		<< %s (%s) has left %s.
opart		<< %s (%s) has left %s %s.
pipe		## Using your adept plumbing skills, you install a "%s" pipe to %s.
pipeclosed	## The "%s" pipe to %s politely disconnects itself.
plain		%s
priv_from	*%s* %s
opriv_from	*%s/%s* %s
priv_to		>%s< %s
public		<%s> %s
quit		<< %s (%s) says "%s" and vanishes!
rehashing	#-#-# The Great Wizard %s casts the rehashing spell on %s.
selfkick	*OUCH!* %s yell "%s" and kick %s out of %s!
selfmode	## You quaff a potion of %s%s.
server		*** %s
serverswitch	## You step into the domain of the Great Wizard %s.
serverinv	## You are currently in contact with: %s
servertime	[DING!] %s claims that it is now %s.
signal		The SIG%s hits!  You die...
sourcedone	## You finish reading the scroll labeled "%s".
sourcestart	## You begin reading the scroll labeled "%s".
stats		## %s
status		[%T] NetHirc: %N on %C (%S) ====================================
switchaway	## The Great Wizard %s raises his staff and says "%s, %s."
time		%a %e %b %Y %T
topic		(%s) %s
topicchange	## %s scrawls something in the dust:
topicinfo	## It is signed by %s and dated %s.
topicpre	## Something is written on the floor here:
traceclass	## Class %d entries linked: %s
traceoper	## Operator number %d is %s.
tweakstart	## You start reading the spellbook labeled "%s".
tweakend	## You finish reading the spellbook labeled "%s".
umodeis		## The ancient runes "%s" are tattooed in glowing letters on your forearm.
userhost	-->	%s
userinfo	There is something written in the dirt here: "Th?s spa?e no? unint?ntio?ally n?t ?eft unbla??."
verboten0	**** Greetings, Citizen!  Effective immediately, all discussion involving:
verboten1	****    %s
verboten2	**** is prohibited.  Have a nice day. 
verboten3	**** Remember, Big Brother is your friend!
verboten4	**** Big Brother objects to the following words:
verboten5	**** Big Brother currently does not object to any words.
version		Son of NetHirc! version %s, %s
wallops		[WALLOPS] -%s- %s
wasnonick	*??* The voice of %s whispers: "I do not recall anyone named %s."
whoischannels	:::: %s is on %s.
whoisidle	:::: %s is %s %s %s.
whoisoperator	:::: %s is wielding %s.
whoisserver	:::: %s is here via %s (%s).
whoisuser	:::: %s is %s@%s or "%s".
whoreply	%-10.10s %-9.9s %-3.3s %s@%s (%s)
whowasuser	:::: %s was %s@%s or "%s".
yourebannedcreep0	## %s blocks your way and says to you:
yourebannedcreep1 	%s
youreoper	## %s hands you %s and says "%s".
