package NetHirc::Handlers;

use strict;

use POSIX qw(strftime);

use NetHirc;
use NetHirc::Channel;
use NetHirc::Formats;
use NetHirc::Helpers;
use NetHirc::Server;
use NetHirc::State;
use NetHirc::StatsDB;

my $rcs_id = q$Id: Handlers.pm,v 1.22 2004/05/04 20:55:46 tony Exp $;
my $rcs_date = q$Date: 2004/05/04 20:55:46 $;

sub on_server {
    my ($conn, $event) = @_;
    server_print($conn, join(' ', $event->args()));
}

sub on_connect { 
    my ($conn, $event) = @_;
    my $terminal = $NetHirc::State::terminal;
    my $deity = $NetHirc::State::deity;
    my $class = $NetHirc::State::class;
    return unless $NetHirc::State::first_connect;
    $NetHirc::State::first_connect = 0;
    log_n_print($conn, 'connect0', $conn->nick());
    log_n_print($conn, 'connect1');
    log_n_print($conn, 'connect2', $deity, $class);
    update_statusline(1);
}

sub on_init1 {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    shift @args;
    my $args = join(' ', @args);
    server_print($conn, $args);
}

sub on_init2 { 
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    if ($NetHirc::State::first_connect) {
	&on_connect($conn, undef);	## Do you smell something?
    }
    shift @args;
    server_print($conn, @args);
}

sub on_motd {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    my $terminal = $NetHirc::State::terminal;
    shift @args;
    log_n_print($conn, 'motd', @args);
}

# XXX fix previous-connect handling for multiple servers.
sub on_endofmotd {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    shift @args;
    my $args = join(' ', @args);
    server_print($conn, $args);
    $NetHirc::State::first_disconnect{$conn} = 1;
    reconnect_old_channels($conn);
    join_commandline_channels($conn);
    update_statusline(0);
}


sub on_nick {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    my ($curnick) = split(/!/, $event->from);
    my $newnick = $args[0];
    my $terminal = $NetHirc::State::terminal;
    my $nickform;
    if ($curnick eq $NetHirc::State::oldnick{$conn}) {	## We've been approved!
	delete $NetHirc::State::oldnick{$conn};
	$nickform = 'mynick';
    } else {
	$nickform = 'nick';
    }
    $terminal->print(ircformat($nickform, $curnick, $newnick));
    logprint($conn, ircformat('nick', $curnick, $newnick));
    update_statusline(0);
}

sub on_nicknameinuse { 
    my ($conn, $event) = @_;
}

sub on_namreply { 
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    my $channel = $args[2];
    my (@namelist) = split(' ', ($event->args)[3]);
    my $sublist;
    my $terminal = $NetHirc::State::terminal;
    if ($channel eq '*') { $channel = "no particular channel"; }
    log_n_print($conn, 'names', scalar(@namelist), $channel);
    while (@namelist) {
	$sublist = join(' ', splice(@namelist, 0, 6));
	log_n_print($conn, 'namelist', $sublist);
    }
}

sub on_cping { 
    my ($conn, $event) = @_;
    my @args = $event->args;
    my $nick = $event->nick;
    my $args = join(" " , @args);
    my $terminal = $NetHirc::State::terminal;
    $irc_formats{'tickle'} = NetHirc::Helpers::pickrandom(@disasters);
    my $tickle_msg = $irc_formats{`tickle`};
    my $text = "PING from $nick: $args";
    log_n_print($conn, 'ctcp', $text);
    $conn->notice($nick, $tickle_msg);
    $conn->ctcp_reply($nick, "PING " . time);
}

sub on_ping {
    my ($conn, $event) = @_;
}

sub on_msg { 
    my ($conn, $event) = @_;
    my ($args, $nick) = (($event->args), $event->nick);
    my $curr_conn = get_current_conn();
    my $terminal = $NetHirc::State::terminal;
    if ($curr_conn eq $conn) {
	$terminal->print(ircformat('priv_from', $nick, $args));
    } else {
	$terminal->print(ircformat('opriv_from', $nick, $conn->server(), $args));
    }
    logprint($conn, ircformat('priv_from', $nick, $args));
    totalitarian_check($args);
    update_counters($conn, $event);
}

sub on_chat { 
    my ($conn, $event) = @_;
#    my ($nick) = $event->nick;
#    my ($to) = $event->to;
#    my ($args) = $event->args;
#    &log_n_print($conn, ircformat('dchat', $nick, $args));
}

sub on_public { 
    my ($conn, $event) = @_;
    my ($nick) = $event->nick;
    my ($to) = $event->to;
    my ($args) = $event->args;
    my $now = time();
    my $t;
    my $u;
    my $terminal = $NetHirc::State::terminal;
    $u = ircformat('ochan', $nick, $to, $args);
    if ($conn ne get_current_conn()) {
	$t = ircformat('oochan', $nick, $to, $conn->server(), $args);
    } elsif (lc($to) ne lc(NetHirc::Channel->current()->name())) {
	$t = $u;
    } else {
	$t = ircformat('public', $nick, $args);
    }
    $terminal->print($t);
    logprint($conn, $u);
    totalitarian_check($args);
    update_counters($conn, $event);
}

sub on_join { 
    my ($conn, $event) = @_;
    my ($channel) = ($event->to)[0];
    my $terminal = $NetHirc::State::terminal;
    log_n_print($conn, 'join', $event->nick(), $event->userhost(), $channel);
    if ($event->nick eq $conn->nick) {	## We tried to join, and it worked.
	NetHirc::Channel->add($channel, $conn);
	NetHirc::Channel->set($channel, $conn);
	update_statusline(0);
    }
}

sub on_part { 
    my ($conn, $event) = @_;
    my ($channel) = ($event->to)[0];
    my $terminal = $NetHirc::State::terminal;
    log_n_print($conn, 'part', $event->nick(), $event->userhost(), $channel);
}

sub on_dcc { 
    my ($conn, $event) = @_;
#    my ($type) = ($event->args)[1];
#    my ($nick) = $event->nick;
#    
#    if (uc($type) eq 'SEND') {
#	if (&confirm(ircformat('admin', 
#				"Receive DCC file from $nick? [yn] "), "y")) {
#	    $conn->new_get($event);
#	}
#    } elsif (uc($type) eq 'CHAT') {
#	if (&confirm(ircformat('admin', 
#				"Commence DCC chat with $nick? [yn] "), "y")) {
#	    $conn->new_chat($event);
#	}
#    } else {
#	&log_n_print($conn, ircformat('error', 
#				"Received unknown DCC type $type from $nick"));
#    }
}

sub on_topic {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    my (@from) = split(/!/, $event->from);
    my $hoser = $from[0];
    my $terminal = $NetHirc::State::terminal;
    ## Now this is schizophrenic.  When the topic changes, the
    ## event looks one way, but when the topic is listed when
    ## entering a channel, it looks different.  Fortunately,
    ## we can tell them apart by what "format" they want to use.
    if ($event->format eq "server") { ## Already here.

	log_n_print($conn, 'topicpre');
	log_n_print($conn, 'topic', @args[1,2]);
    } elsif ($event->format eq "topic") { ## topic change!
	log_n_print($conn, 'topicchange', $hoser);
	log_n_print($conn, 'topic', ($event->to)[0], $args[0]);
    }
}

sub on_topicinfo {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    my $terminal = $NetHirc::State::terminal;
    my $time = timeformat($args[3]);
    log_n_print($conn, 'topicinfo', $args[2], $time);
}

sub on_cversion {
    my ($conn, $event) = @_;
    my $hoser = $event->nick;
    my @args = $event->args();
    my $terminal = $NetHirc::State::terminal;
    $irc_formats{'tickle'} = NetHirc::Helpers::pickrandom(@disasters);
    my $tickle_msg = $irc_formats{'tickle'};
    log_n_print($conn, 'ctcp', "VERSION from $hoser");
    my $text = ircformat('version', $NetHirc::VERSION, $rcs_date);
    chomp $text;
    $conn->notice($hoser, $tickle_msg);
    $conn->ctcp_reply($hoser, "VERSION $text");
}

sub on_nosuchnick {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my $terminal = $NetHirc::State::terminal;
    log_n_print($conn, 'nosuchnick', $args[1]);
}

sub on_mode {
    my ($conn, $event) = @_;
    my $to = ($event->to)[0];
    my @args = $event->args;
    my $nick = $event->nick;
    my $modestring = shift @args;
    my @modes;
    my @t;
    NetHirc::Helpers::modesplitter(\@modes, $modestring, @args);
    my $targets = join(' ', @args);
    $targets = " " . $targets if ($targets);	## YAPPH.
    $targets =~ s/\s*$//;
    ## Channel-affecting modes are "to" the channel.  (incl. chanops)
    ## Person-affecting modes are "to" the person.  But only the person
    ## sending them actually receives them, so...
    if ($to eq $nick) {	## Personal mode.
	log_n_print($conn, 'selfmode', $modestring);
	for my $k (@modes) {
	    log_n_print($conn, 'plain', $self_mode_comments{$k->[0]});
	}
    } else { ## Channel mode.
	log_n_print($conn, 'channelmode', $to, $nick, $modestring, $targets);
	for my $k (@modes) {
	    log_n_print($conn, 'plain', sprintf($channel_mode_comments{$k->[0]}, $to, $k->[1]));
	}
    }
}

sub on_nosuchchannel {
    my ($conn, $event) = @_;
    my $channel = ($event->args)[1];
    log_n_print($conn, 'nosuchchannel', $channel);
}

sub on_channelmodeis {
    my ($conn, $event) = @_;
    my @args = $event->args;
    log_n_print($conn, 'channelmodeis', @args[1,2]);
}

sub on_channelcreate {
    my ($conn, $event) = @_;
    my @args = $event->args;
    log_n_print($conn, 'channelcreate', $args[1], &timeformat($args[2]));
}

sub on_kick {
    my ($conn, $event) = @_;
    my $to = ($event->to)[0];
    my $from = $event->from;
    my @args = $event->args;
    my $fromnick = $from;
    my @formargs;
    my @t;
    $fromnick =~ s/!.*//;
    if ($to eq $conn->nick) {	## Ow, that was me!
	if ($fromnick eq $conn->nick) {	## Ow, that was my foot kicking me!
	    @formargs = +('selfkick', "You", $args[1], "yourself", $args[0]);
	} else {
	    @formargs = +('kick', $fromnick , $args[1], "you", $args[0]);
	}
	## In either event...
	log_n_print($conn, @formargs);
	NetHirc::Channel->remove($args[0], $conn);
	if (scalar NetHirc::Channel->list() == 0) {
	    log_n_print($conn, 'lastchannel');
	    log_n_print($conn, 'nochannels');
	    NetHirc::Channel->nochannel($conn);
	} else {
	    log_n_print($conn, 'nextchannel', NetHirc::Channel->current()->name());
	}
    } else { 	## Poor sod, it was someone else.
	if ($fromnick eq $conn->nick) {	## Poor sod, that was my foot. :-)
	    @formargs = +('selfkick', "You", $args[1], $to, $args[0]);
	} else {
	    @formargs = +('kick', $fromnick, $args[1], $to, $args[0]);
	}
	log_n_print($conn, @formargs);
    }
}

sub on_caction {
    my ($conn, $event) = @_;
    my $from = $event->from;
    my @args = $event->args;
    my $to = ($event->to)[0];
    my $terminal = $NetHirc::State::terminal;
    my $now = time();
    my $action;
    my ($t, $u);
#    shift @args;
    $from =~ s/!.*//;
    $event->dump if $conn->{_debug};
    $action = join(' ', @args);
    if ($conn ne get_current_conn()) {
	$t = ircformat('ooaction', $from, $to, $conn->server(), $action);
    } elsif (lc($to) ne lc(NetHirc::Channel->current()->name())) {
	$t = $u;
    } else {
	$t = ircformat('action', $from, $action);
    }
    $terminal->print($t);
    logprint($conn, ircformat('action', $from, $to, $action));
    totalitarian_check($action);
}

sub on_liststart {
    my ($conn, $event) = @_;
    log_n_print($conn, 'liststart', $event->from);
}

sub on_list {
    my ($conn, $event) = @_;
    my $from = $event->from;
    my @args = $event->args;
    shift @args;		## The first one was our name anyway.
    my ($channel, $users, $topic) = @args;
    my $format = 'list';
    if ($topic) {
	$topic = qq("$topic");
    } else {
	$topic = "nothing in particular";
    }
    if ($users == 1) {
	$format = 'list1';
    }
    log_n_print($conn, $format, $channel, $users, $topic);
}

sub on_listend {
    my ($conn, $event) = @_;
    log_n_print($conn, 'listend', $event->from);
}

sub on_stats {
    my ($conn, $event) = @_;
    my @args = $event->args;
    shift @args;		## first arg is our name.
    log_n_print($conn, 'stats', join(':', @args));
}

sub on_whoisuser {
    my ($conn, $event) = @_;
    my @args = $event->args;
    shift @args;
    splice @args, 3, 1;
    log_n_print($conn, 'whoisuser', @args);
}

sub on_whoisserver {
    my ($conn, $event) = @_;
    my @args = $event->args;
    shift @args;
    log_n_print($conn, 'whoisserver', @args);
}

sub on_whoisoperator {
    my ($conn, $event) = @_;	
    my @args = $event->args;
    shift @args;
    my $hoser = $args[0];
    my $weapon = select_weapon($hoser);
    log_n_print($conn, 'whoisoperator', $hoser, $weapon);
}

sub on_whoisidle {
    my ($conn, $event) = @_;
    my @args = $event->args;
    log_n_print($conn, 'whoisidle', @args[1,2,4], scalar(localtime($args[3])));
}

sub on_whoischannels {
    my ($conn, $event) = @_;
    my @args = $event->args;
    log_n_print($conn, 'whoischannels', @args[1,2]);
};

sub on_n_local {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my $from = $event->from;
    server_print($conn, $args[1]);
}

sub on_n_global {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my $from = $event->from;
    server_print($conn, $args[1]);
}

sub on_cannotsendtochan {
    my ($conn, $event) = @_;
    my @args = $event->args;
    log_n_print($conn, 'cannotsendtochan', $args[1]);
}

sub on_whoreply {
    my ($conn, $event) = @_;
    my @args = $event->args;
    ## order of args: channel login host server nick flags ircname
    ##                1       2     3    4      5    6     7
    log_n_print($conn, 'whoreply', @args[1,5,6,2,3,7]);
}

sub on_disconnect {
    my ($conn, $event) = @_;
    ## We've been cut off!
    my $from = $event->from;
    my $reason = ($event->args)[0];
    my @t;
    NetHirc::Channel->disconnect($conn);
    log_n_print($conn, 'disconnect', $from, $reason);
    if (scalar NetHirc::Channel->list() == 0) {
	log_n_print($conn, 'nochannels');
    }
    NetHirc::Server->remove($conn->server);
    update_statusline(0);
}

sub on_leaving {	
    my ($conn, $event) = @_;
    my $args = $NetHirc::State::bailmsg;
    return unless $NetHirc::State::first_disconnect{$conn};
    log_n_print($conn, 'quit', $conn->nick, "i.e. you", $args);
    $NetHirc::State::first_disconnect{$conn} = 0;
}

sub on_quit {
    my ($conn, $event) = @_;	
    my ($args) = $event->args;
    my $from = $event->from;
    my ($fromnick, $fromaddr) = split(/!/, $from);
    ## They're gone.  Sigh.
    log_n_print($conn, 'quit', $fromnick, $fromaddr, $args);
}

sub on_kill {
    my ($conn, $event) = @_;	
    ## Oh man, that's *gotta* hurt!
    my $hoser = (split(/!/, $event->from))[0];
    my $weapon = select_weapon($hoser);
    log_n_print($conn, 'kill', $hoser, $weapon);
}

sub on_csource {
    my ($conn, $event) = @_;	
    my $hoser = (split(/!/, $event->from))[0];
    my @args = $event->args();
    ## ircII doesn't respond to this one.
    log_n_print($conn, 'ctcp', "SOURCE from $hoser");
    my $text = ircformat('csource');
    chomp $text;
    $conn->ctcp_reply($hoser, "SOURCE $text");
}

sub on_ctime {
    my ($conn, $event) = @_;	
    my $hoser = (split(/!/, $event->from))[0];
    my @args = $event->args;
    ## ircII sends back scalar localtime, or something like that.
    log_n_print($conn, 'ctcp', "TIME from $hoser");
    $conn->ctcp_reply($hoser, "TIME " . scalar localtime);
}

sub on_cuserinfo {
    my ($conn, $event) = @_;	
    my $hoser = (split(/!/, $event->from))[0];
    my @args = $event->args;
    ## ircII sends a mostly-blank thing back.
    log_n_print($conn, 'ctcp', "USERINFO from $hoser");
    my $text = ircformat('userinfo');
    chomp $text;
    $conn->ctcp_reply($hoser, "USERINFO $text");
}

sub on_clientinfo {
    my ($conn, $event) = @_;	
    my $hoser = (split(/!/, $event->from))[0];
    my @args = $event->args;
    ## ircII sends back a list of supported CTCP's.
    log_n_print($conn, 'ctcp', "CLIENTINFO from $hoser");
    my $text = ircformat('clientinfo', $NetHirc::State::supported_ctcp);
    chomp $text;
    $conn->ctcp_reply($hoser, "CLIENTINFO $text");
}

sub on_cfinger {
    my ($conn, $event) = @_;	
    my $hoser = (split(/!/, $event->from))[0];
    my @args = $event->args;
    ## ircII does a pseudo-finger with real name and idle time.
    log_n_print($conn, 'ctcp', "FINGER from $hoser");
    my $text = ircformat('finger', $conn->nick(), $NetHirc::State::class);
    chomp $text;
    $conn->ctcp_reply($hoser, "FINGER $text");
}

sub on_crversion {
    my ($conn, $event) = @_;	
    my $hoser = (split(/!/, $event->from))[0];
    my $args = join(' ', $event->args);
    ## ircII sends back something vaguely uname-ish and some other cruft.
    log_n_print($conn, 'ctcp', "VERSION reply from $hoser: $args");
}

sub on_crsource {
    my ($conn, $event) = @_;	
    my $hoser = (split(/!/, $event->from))[0];
    my $args = join(' ', $event->args);
    ## Anybody?
    log_n_print($conn, 'ctcp', "SOURCE reply from $hoser: $args");
}

sub on_crtime {
    my ($conn, $event) = @_;	
    my $hoser = (split(/!/, $event->from))[0];
    my $args = join(' ', $event->args);
    log_n_print($conn, 'ctcp', "TIME reply from $hoser: $args");
}

sub on_cruserinfo {
    my ($conn, $event) = @_;	
    my $hoser = (split(/!/, $event->from))[0];
    my $args = join(' ', $event->args);
    log_n_print($conn, 'ctcp', "USERINFO reply from $hoser: $args");
}

sub on_crclientinfo {
    my ($conn, $event) = @_;	
    my $hoser = (split(/!/, $event->from))[0];
    my $args = join(' ', $event->args);
    log_n_print($conn, 'ctcp', "CLIENTINFO reply from $hoser: $args");
}

sub on_crfinger {
    my ($conn, $event) = @_;	
    my $hoser = (split(/!/, $event->from))[0];
    my $args = join(' ', $event->args);
    log_n_print($conn, 'ctcp', "FINGER reply from $hoser: $args");
}

sub on_crping { 
    my ($conn, $event) = @_;
    my @args = $event->args;
    my $hoser = (split(/!/, $event->from))[0];
    my $secs = (split(' ', $args[0]))[1];
    my $delay = $secs - $NetHirc::State::cpings{$hoser};
    log_n_print($conn, 'ctcp', "PING reply from $hoser: $delay seconds");
    delete $NetHirc::State::cpings{$hoser};
}

sub on_wasnosuchnick {
    my ($conn, $event) = @_;
    my $oopsnick = ($event->args)[1];
    my ($from) = $event->from;
    log_n_print($conn, 'wasnonick', $from, $oopsnick);
}

sub on_youreoper {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    my ($from) = $event->from;
    my $hoser = $conn->nick();
    my $weapon = select_weapon($hoser);
    log_n_print($conn, 'youreoper', $from, $args[1]);
}

sub on_umodeis {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    log_n_print($conn, 'umodeis', $args[1]);
}

sub on_whowasuser {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    log_n_print($conn, 'whowasuser', @args[1,2,3,5]);
}

sub on_adminme {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    log_n_print($conn, 'adminme', $args[1]);
}

sub on_adminloc1 {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    log_n_print($conn, 'adminloc', $args[1]);
}

sub on_adminloc2 {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    log_n_print($conn, 'adminloc', $args[1]);
}

sub on_adminemail {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    log_n_print($conn, 'adminemail', $args[1]);
}

sub on_inviting {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    log_n_print($conn, 'inviting', @args[1,2]);
}

sub on_invite {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    my ($from) = $event->from;
    my ($hoser) = (split(/!/, $from))[0];
    log_n_print($conn, 'invite', $hoser, $args[0]);
}

sub on_traceoper {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    log_n_print($conn, 'traceoper', @args[2,3]);
}

sub on_traceclass {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    log_n_print($conn, 'traceclass', @args[2,3]);
}

sub on_banlist {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    log_n_print($conn, 'banlist', @args[1..3], scalar gmtime $args[4]);
}

sub on_away {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    shift @args;
    log_n_print($conn, 'away', @args);
}

sub on_links {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    shift @args;
    log_n_print($conn, 'links', @args);
}

sub on_ison {
    my ($conn, $event) = @_;
    my (@args) = $event->args;
    my $t;
    if ($args[1]) {
	log_n_print($conn, 'ison', $args[1]);
    } else {
	log_n_print($conn, 'isnton');
    }
}

sub on_notexttosend {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my ($from) = $event->from;
    log_n_print($conn, 'notexttosend', $from, $args[1]);
}

sub on_yourebannedcreep {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my ($from) = $event->from;
    log_n_print($conn, 'yourebannedcreep0', $from);
    log_n_print($conn, 'yourebannedcreep1', $args[1]);
}

sub on_goof {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my ($from) = $event->from;
    log_n_print($conn, 'goof', $from, $args[$#args]);
}

sub on_needmoreparams {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my ($from) = $event->from;
    log_n_print($conn, 'needmoreparams', $args[1]);
}

sub on_userhost {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my @t;
    for my $i (split(' ', $args[1])) {
	log_n_print($conn, 'userhost', $i);
    }
}

sub on_nosuchserver {
    my ($conn, $event) = @_;
    my @args = $event->args;
    log_n_print($conn, 'nosuchserver', $args[1]);
}

sub on_unknowncommand {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my ($from) = $event->from;
    log_n_print($conn, 'goof', $from, $args[2]);
}

sub on_rehashing {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my ($from) = $event->from;
    my $rthingies = join(' ', @args[1..($#args - 1)]);
    log_n_print($conn, 'rehash', $from, $rthingies);
}

sub on_notice {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my ($from) = split(/!/, $event->from);
    log_n_print($conn, 'notice', $from, $args[0]);
    update_counters($conn, $event);
}

sub on_wallops {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my ($from) = $event->from;
    log_n_print($conn, 'wallops', $from, $args[0]);
    update_counters($conn, $event);
}

sub on_snotice {
    my ($conn, $event) = @_;
    my @args = $event->args;
    log_n_print($conn, 'plain', $args[0]);
}

sub on_toomanychannels {
    my ($conn, $event) = @_;
    my @args = $event->args;
    log_n_print($conn, 'toomanychannels');
}

sub on_useronchannel {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my ($from) = $event->from;
    log_n_print($conn, 'goof', $from, sprintf("%s %s %s", @args[1,3,2]));
}

sub on_cantkillserver {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my ($from) = $event->from;
    log_n_print($conn, 'goof0', $from, $args[$#args]);
}

sub on_notopic {
    my ($conn, $event) = @_;
    my @args = $event->args;
    log_n_print($conn, 'notopic', $args[1]);
}

sub on_time {
    my ($conn, $event) = @_;
    my @args = $event->args;
    log_n_print($conn, 'servertime', @args[1,2]);
}

sub on_switchaway {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my ($from) = $event->from;
    log_n_print($conn, 'switchaway', $from, @args);
}

sub on_nickgoof {
    my ($conn, $event) = @_;
    my @args = $event->args;
    my ($from) = $event->from;
    log_n_print($conn, 'goof', $from, $args[$#args]);
}

sub on_cdcc {
    my ($conn, $event) = @_;
}

sub dcc_open {
    my ($conn, $event) = @_;	
}

sub dcc_update {
    my ($conn, $event) = @_;	
}

sub dcc_close {
    my ($conn, $event) = @_;	
}

sub on_ignore { 
    my ($conn, $event) = @_;
    ## Wow, this is really exciting!
}

sub on_nomotd {
    my ($conn, $event) = @_;
    my $server = $conn->server;
    my $terminal = $NetHirc::State::terminal;
    log_n_print($conn, 'nomotd', $server);
    $NetHirc::State::first_disconnect{$conn} = 1;
    reconnect_old_channels($conn);
    join_commandline_channels($conn);
    update_statusline(0);
}

sub on_unknown {
    my ($conn, $event) = @_;
    $event->dump;
}

#############################
##### Terminal Handlers #####
#############################

sub term_msg { 
    my ($terminal, $event) = @_;
    my ($recipient, $text) = split(' ', $event->args(), 2);
    my $t = ircformat('priv_to', $recipient, $text);
    my $conn = NetHirc::Server->current()->conn();
    if (NetHirc::Channel->is_channel_name($recipient)) {
	$terminal->print(ircformat('Emsg'));
	return;
    }
    $conn->privmsg($recipient, $text);
    if ($NetHirc::State::echo) { 
	$terminal->print($t); 
    }
    logprint($conn, $t);
    totalitarian_check($text);
}

sub term_ctcp { 
    my ($terminal, $event) = @_;
    my ($target, $type, $evenmore) = split(' ', $event->args(), 3); # Consistent w/ircII
    my $conn = NetHirc::Server->current()->conn();
    $conn->ctcp($type, $target, $evenmore);
    $type = uc($type);
    my $t = ircformat('ctcp', "$type sent to $target");
    if ($type eq 'PING') {
	$NetHirc::State::cpings{$target} = time();
    }
    $terminal->print($t);
    logprint($conn, $t);
}

#sub term_dcc { 
#    my ($conn, $first, $rest) = @_;
#    my ($type, $target, $evenmore) = split(' ', $rest, 3);
#    my ($address, $port);
#    $type = uc($type);
#    if ($type eq 'CHAT') {
#	## Magically select an address and port.  The DCC spec seems
#	## to indicate stupid tricks with port 0 and INADDR_ANY.
#	$conn->new_chat(1, $target, INADDR_ANY, 0);
#	## This object is added to the list of sockets.  We need to
#	## maintain it on our list of channels though.
#	&admin_print("You are covered in ice!  The ice doesn't feel cold!");
#    } elsif ($type eq 'GET') {
#	## Check the list of files that have been offered to us,
#	## and snarf this one if it is there.  The list will be
#	## indexed by nick or filename, perhaps...
#	&admin_print("The blast of fire hits you!  Yon don't feel hot!");
#    } elsif ($type eq 'SEND') {
#	## Put this on a send queue.  The rest is handled in a handler.
#	&admin_print("The rust monster hits you!  Your boots are unaffected!");
#    } else {
#	&admin_print("The imp casts aspersions on your ancestry.");
#    }
#}

sub term_eval { 
    my ($terminal, $event) = @_;
    eval $event->args();
    if ($@) {
	$terminal->print(ircformat('Eeval', $@));
    }
}


sub term_cmd_char { 
    my ($terminal, $event) = @_;
    my $args = $event->args();
    my ($cc) = (split(' ', $args))[0];
    $terminal->param('cmdchar', $cc);
}

sub term_system { 
    my ($terminal, $event) = @_;
    my $rc;
    $rc = system($event->args());	## It is not my place to protect you.
    if ($rc != 0) {
	$terminal->print(ircformat('Esystem0'));
	$terminal->print(ircformat('Esystem1', $rc));
    };
}

## XXX fix saytoer to handle multiple servers
sub term_pipe { 
    my ($terminal, $event) = @_;
    my $fh = new IO::File;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    my $irc = $NetHirc::State::irc;
    if (not NetHirc::Channel->current()) {
	$terminal->print(ircformat('Epipesay'));
	return;
    }
    unless ($fh->open("$rest |")) {
	$terminal->print(ircformat('Epipe', $!));
	return;
    }
    $irc->addfh($fh, NetHirc::Helpers::saytoer($conn, NetHirc::Channel->current(), $fh, $rest));
    $terminal->print(ircformat('pipe', $rest, NetHirc::Channel->current()->name()));
}

sub term_pipemsg { 
    my ($terminal, $event) = @_;
    my $fh = new IO::File;
    my ($recipient, $cmd) = split(' ', $event->args(), 2);
    my $conn = NetHirc::Server->current()->conn();
    if (not $recipient) {
	$terminal->print(ircformat('Epipesay'));
	return;
    }
    unless ($fh->open("$cmd |")) {
	$terminal->print(ircformat('Epipe', $!));
	return;
    }
    $NetHirc::State::irc->addfh($fh, &saytoer($conn, $recipient, $fh, $cmd));
    $terminal->print(ircformat('pipe', $cmd, $recipient));
}

sub term_quit { 
    my ($terminal, $event) = @_;
    my $rest = $event->args() || "You escaped the dungeon.";
    my $bailmsg = $event->type() eq 'pickle' ? "Let's pickle!" : $rest;
    $NetHirc::State::bailmsg = $rest;
    map { $_->conn()->quit($bailmsg) } NetHirc::Server->list();
    die "term_quit";
}

sub term_echo { 
    my ($terminal, $event) = @_;
    my $first = $event->type();
    my $rest = $event->args();
    if (lc($rest) eq 'on') {
	$terminal->print(ircformat('echo_on'));
	$NetHirc::State::echo = 1;
    } elsif (lc($rest) eq 'off') {
	$terminal->print(ircformat('echo_off'));
	$NetHirc::State::echo = 0;
    } else {
	$terminal->print(ircformat('Eecho', uc $first, $first, $first));
    }
}

sub term_admin { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    $conn->admin($event->args());
}

sub term_server { 
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    if (not $rest) {
	my @srvnames = map { $_->name() } NetHirc::Server->list();
	$terminal->print(ircformat('serverinv', join(" ", @srvnames)));
	return;
    }
    my ($curr) = grep { $rest eq $_->server() } NetHirc::Channel->list();
    if ($curr) {
	NetHirc::Server->set($rest);
	NetHirc::Channel->set($curr->name(), NetHirc::Server->current()->conn());
	update_statusline(0);
	return;
    }
    my $irc = $NetHirc::State::irc;
    my $srv = NetHirc::Server->current();
    my $conn;
    my ($server, $nick, $username, $ircname) = split(' ', $rest, 4);
    if (not $srv) {
	$nick = $NetHirc::State::options{'n'};
	$ircname = $NetHirc::State::options{'i'};
	$username = $NetHirc::State::options{'u'};
    } else {
	my $conn = $srv->conn();
	$nick = $nick || $conn->nick();
	$ircname = $ircname || $conn->ircname();
	$username = $username || $conn->username();
    }
    my $newconn = $irc->newconn( Nick => $nick,
			      Ircname => $ircname,
			      Username => $username,
			      Server => $server );
    ## Rejoining old channels is done at endofmotd, since we know
    ## we're connected there.
    return unless $newconn;
    NetHirc::Server->add($newconn);
    NetHirc::Server->set($newconn->server());
    NetHirc::Channel->nochannel($newconn);
    update_statusline(0);
}

sub term_ignore { 
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my ($type, $masks) = split(' ', $rest, 2);
    my $conn = NetHirc::Server->current()->conn();
    unless (grep { $type eq $_ } 
	    qw(public msg ctcp notice channel nick other all)) {
	$terminal->print(ircformat('Eignore'));
	return;
    }
    $conn->ignore($type, $masks);
}

sub term_info { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    $conn->info($rest);
}

sub term_invite { 
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my ($target, $more) = &firstword($rest);
    my $tchan;
    my $conn = NetHirc::Server->current()->conn();
    if ($more =~ /^[#&]/) {
	($tchan) = &firstword($more);
    } else {
	$tchan = NetHirc::Channel->current()->[0];
    }
    $conn->invite($target, $tchan);
}

sub term_ison { 
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my $conn = NetHirc::Server->current()->conn();
    unless ($rest) {
	$terminal->print(ircformat('Eison'));
	return;
    }
    $conn->ison($rest);
}

sub term_kick {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my ($channel, $target, $more) = split(' ', $rest, 3);
    my $conn = NetHirc::Server->current()->conn();
    unless (grep { $_->name() eq $channel } NetHirc::Channel->list()) {
	$terminal->print(ircformat('Enotinchannel', $channel));
	return;
    } 
    $conn->kick($channel, $target, $more);
}

sub term_mode { 
    my ($terminal, $event) = @_;
    my ($target, $mode, $args);
    my $rest = $event->args();
    my $conn = NetHirc::Server->current()->conn();
    ($target, $mode, $args) = split(' ', $rest, 3);
    $conn->mode($target, $mode, $args);
}

sub term_op {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my (@nicks) = split $rest;
    my ($mode, @chunk);
    my $conn = NetHirc::Server->current()->conn();
    while (@nicks) {
	@chunk = splice(@nicks, 0, 4);
	$mode = "+" . ("o" x @chunk);
	$conn->mode(NetHirc::Channel->current()->[0], $mode, @chunk);
    }
}

sub term_links { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    $conn->links(split $rest);		## Connection.pm says to read the RFC.
}

sub term_list { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    if ($rest) {
	&term_list_0($conn, $rest);
    } else {
	$terminal->confirm($irc_formats{'Clist'}, "y", sub { term_list_0($conn, $rest) } );
    }
}

sub term_list_0 {
    my ($conn, $rest) = @_;
    $conn->list($rest);
}

sub term_lusers { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    $conn->lusers($rest);
}

sub term_me { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    unless (NetHirc::Channel->current()) {
	$terminal->print(ircformat('Eme'));
	return;
    }
    my $t = ircformat('action', $conn->nick, $rest);
    my $u = ircformat('oaction', $conn->nick, NetHirc::Channel->current()->name(), $rest);
    $conn->me(NetHirc::Channel->current()->name(), $rest);
    if ($NetHirc::State::echo) { 
	$terminal->print($t); 
    } 
    logprint($conn, $u); 
}

sub term_motd {
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    $conn->motd($rest);
}

sub term_nick { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    $NetHirc::State::oldnick{$conn} = $conn->nick;
    my $rest = $event->args();
    $conn->nick($rest);
}

sub term_oper { 
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my (@words) = split(' ', $rest);
    my ($mynick, $operpasswd);
    my $conn = NetHirc::Server->current()->conn();
    if (@words < 1) {
	### BZZZT
    } elsif (@words == 1) {
	$terminal->print(ircformat('Woperpasswd'));
	$operpasswd = $words[0];
	$mynick = $conn->nick;
    } else {
	($mynick, $operpasswd) = @words;
    }
    $conn->oper($mynick, $operpasswd);
}

## XXX announce where we're talking after leaving channels
sub term_leave { 
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my (@channels) = split(' ', $rest);
    my $conn = NetHirc::Server->current()->conn();
    unless (@channels) {
#	$terminal->print(ircformat('Epart'));		## ircII behaviour.
#	return;
	$terminal->print(ircformat('Wpart', NetHirc::Channel->current()->name()));
	NetHirc::Channel->remove(NetHirc::Channel->current()->name(), $conn);
    }
    my @newchan;
    for my $c (@channels) {
	unless (grep { lc($c) eq lc($_->name()) } NetHirc::Channel->list()) {
	    $terminal->print(ircformat('Enotinchannel', $c));
	}
    }
    $conn->part(@channels);
    for my $c (@channels) {
	NetHirc::Channel->remove($c, $conn);
    }

    unless ((NetHirc::Channel->list())[0]) {
	$terminal->print(ircformat('nochannels'));
    } else {
	fake_join_or_switch(NetHirc::Channel->current());
    }
    update_statusline(0);
}

sub term_sconnect { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    my @words = split(' ', $rest);
    unless (@words > 1) {
	$terminal->print(ircformat('Esconnect'));
	return;
    }
    $conn->sconnect($rest);
}

sub term_squit { 
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my ($server, $comment) = split(' ', $rest, 2);
    my $conn = NetHirc::Server->current()->conn();
    unless ($server) {
	my $monster = pickrandom(@ordinary_monsters);
	$terminal->print(ircformat('Esquit0', $monster, &pickrandom(@scrolls)));
	$terminal->print(ircformat('Esquit1', $monster));
	return;
    }
    $conn->squit(split $rest);
}

sub term_stats { 
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my ($statbits, $server) = split(' ', $rest, 2);
    my $conn = NetHirc::Server->current()->conn();
    unless ($statbits) {
	$terminal->print(ircformat('Estats'));
	return;
    }
    $conn->stats($statbits, $server);
}

sub term_time { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    $conn->time($rest);
}

sub term_topic { 
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my ($channel, $topic) = split(' ', $rest, 2);
    my $conn = NetHirc::Server->current()->conn();
    $conn->topic($channel, $topic);			## no defaults!
}

sub term_trace { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    $conn->trace($rest);
}

sub term_tweak {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    $terminal->print(ircformat('tweakstart', $rest));
    my $rc = do $rest;
    unless ($rc) 
    {
	# Esource format
	if ($@) {
	    $terminal->print(ircformat('Etweak', $@));
	}
	unless (defined $rc)
	{
	    $terminal->print(ircformat('Etweak', $!));
	}
    }
    $terminal->print(ircformat('tweakend', $rest));
}

sub term_userhost { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    unless ($rest) {
	$terminal->print(ircformat('Euserhost'));
    }
    $conn->userhost($rest);
}

sub term_users { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    $conn->users($rest);
}

sub term_version { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    $conn->version($rest);
}

sub term_wallops { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    unless ($rest) {
	$terminal->print(ircformat('Ewallops'));
	return;
    }
    $conn->wallops($rest);
}

sub term_who { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    $conn->who($rest);
}

sub term_whois { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    unless ($rest) {
	$terminal->print(ircformat('Ewhois'));
    }
    $conn->whois($rest);
}

sub term_whowas { 
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    unless ($rest) {
	$terminal->print(ircformat('Ewhowas'));
    }
    $conn->whowas($rest);
}

sub term_say {
    my ($terminal, $event) = @_;
    my $destchan = NetHirc::Channel->current();
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    chomp($rest);
    if (not $destchan or $destchan->name() eq NetHirc::Channel::NO_CHANNEL) {
	$terminal->print(ircformat('Esay'));
	return;
    }
    my $t = ircformat('public', $conn->nick, $rest);
    my $u = ircformat('ochan', $conn->nick, $destchan->name(), $rest);
    $conn->privmsg($destchan->name(), $rest);
    if ($NetHirc::State::echo) { 
	$terminal->print($t); 
    }
    logprint($conn, $u);
    totalitarian_check($rest);
}

sub term_reverse {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my $reversed = scalar reverse($rest);
    my $conn = NetHirc::Server->current()->conn();
    &term_say($conn, 'say', $reversed);
}

sub term_key {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my ($channel, @other) = split $rest;
    my $conn = NetHirc::Server->current()->conn();
    $conn->mode($channel, '+k', @other);
}

sub term_ban {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my ($channel, @other) = split $rest;
    my $conn = NetHirc::Server->current()->conn();
    $conn->mode($channel, '+b', @other);
}

sub term_unban {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my ($channel, @other) = split $rest;
    my $conn = NetHirc::Server->current()->conn();
    $conn->mode($channel, '-b', @other);
}

sub term_kickban {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my ($channel, @other) = split $rest;
    my $conn = NetHirc::Server->current()->conn();
    $conn->mode($channel, '+b', @other);
    $conn->kick($channel, @other);
}

sub term_notice {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my ($target, $text) = split(' ', $rest, 2);
    my $t = ircformat('notice', $target, $text);
    my $conn = NetHirc::Server->current()->conn();
    $conn->notice($target, $text);
    if ($NetHirc::State::echo) { 
	$terminal->print($t); 
    }
    logprint($conn, $t);
}

sub term_names {
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    if ($rest) {
	&term_names_0($conn, $rest);
    } else {
	$terminal->confirm($irc_formats{'Cnames'}, "y", sub { term_names_0($conn, $rest) } );
    }
}

sub term_names_0 {
    my ($conn, $rest) = @_;
    $conn->names($rest);
}

sub term_channels {
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my @channels = NetHirc::Channel->list();
    @channels = grep { $_->[0] ne NetHirc::Channel::NO_CHANNEL } @channels;
    unless (scalar @channels) {
	NetHirc::Helpers::admin_print("You are empty-handed.");
	return;
    }
    $terminal->print(ircformat('channels', join(' ', map { $_->[0] } @channels)));
}

sub term_nothing {
    ## Hey!  Let's do nothing!
}

sub term_join_or_switch {
    my ($terminal, $event) = @_;
    my $channel;
    my $server = NetHirc::Server->current();
    my $conn = $server->conn();
    my $rest = $event->args();
    ($channel = $rest) =~ s/\s.*//;
    if (not NetHirc::Channel->is_channel_name($channel)) {
	$terminal->print(ircformat('Ebadchannel'));
	return;
    } 
    my @net = grep { lc($_->name()) eq lc($channel) } NetHirc::Channel->list();
    if (@net == 1) {
	## Already here, eh?
	NetHirc::Channel->set($net[0]->name());
	NetHirc::Server->set($net[0]->server());
	$terminal->print(ircformat('chanswitch', $net[0]->name()));
	update_statusline(0);
	return;
    }
    $terminal->print(ircformat('joining', $channel));
    $conn->join($channel);
    update_statusline(0);
}

sub term_quote {
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    $conn->sl($rest);
}

sub term_happyslash {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    if ($rest) {
	$terminal->param('happyslash' ,&get_boolean($rest));
    }
    $terminal->print(ircformat('hsstatus', &print_boolean($terminal->param('happyslash'), "disabled", "enabled")));
}

sub term_debug {
    my ($terminal, $event) = @_;
    my $bool;
    my $irc = $NetHirc::State::irc;
    my $rest = $event->args();
    if ($rest) {
	$bool = &get_boolean($rest);
	$irc->debug($bool);
	NetHirc::Server->debug($bool);
	## This should be good enough for now; do we want to modify
	## Net::IRC to recurse through all its Connection objects
	## and turn their debugging on when its debugging is turned
	## on?  It's a touchy-feely preference thingy...
    }
}

sub term_away {
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    $conn->away($rest);
}

sub term_rehash {
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    $conn->rehash($rest);
}

sub term_restart {
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    $conn->restart;
}

sub term_summon {
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    unless ($rest) {
	$terminal->print(ircformat('Esummon'));
	return;
    }
    $conn->summon($rest);
}

sub term_yoda {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my $yoda = NetHirc::Helpers::yoda_front($rest);
    $event->{'Args'} = $yoda;
    &term_say($terminal, $event);
}

sub term_yodamsg {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my ($recipient, $text) = split(' ', $rest, 2);
    my $yoda = NetHirc::Helpers::yoda_front($text);
    $event->{'Args'} = "$recipient $yoda";
    &term_msg($terminal, $event);
}	

sub term_pigl {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my $pigl = NetHirc::Helpers::pigl_front($rest);
    $event->{'Args'} = $pigl;
    &term_say($terminal, $event);
}

sub term_piglmsg {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my ($recipient, $text) = split(' ', $rest, 2);
    my $pigl = NetHirc::Helpers::pigl_front($text);
    $event->{'Args'} = "$recipient $pigl";
    &term_msg($terminal, $event);
}	

sub term_mirror {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my $mirror = NetHirc::Helpers::mirror($rest);
    $event->{'Args'} = $mirror;
    &term_say($terminal, $event);
}

sub term_mirrormsg {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my ($recipient, $text) = split(' ', $rest, 2);
    my $mirror = NetHirc::Helpers::mirror($text);
    $event->{'Args'} = "$recipient $mirror";
    &term_msg($terminal, $event);
}

sub term_log {
    my ($terminal, $event) = @_;
    my $logfh;
    my $logname;
    my $rest = $event->args();
    my $bool = &get_boolean($rest);
    my $form = $irc_formats{'logfilename'};
    chomp $form;
    $logname = sprintf($form, time(), $$);
    if ($bool) {
	if ($NetHirc::State::log) {
	    $terminal->print(ircformat('Elogging'));
	    return;
	}
	$logfh = new IO::File(">$logname");
	if ($logfh) {
	    $NetHirc::State::log = $logfh;
	    $NetHirc::State::logname = $logname;
	    $NetHirc::State::log->autoflush(1);
	    $NetHirc::State::log->print(ircformat('logstarttext', $$, scalar(gmtime(time()))));
	    $terminal->print(ircformat('logstart0'));
	    $terminal->print(ircformat('logstart1', $logname));
	} else {
	    $terminal->print(ircformat('Ecantlog', $!));
	}
    } else {
	if ($NetHirc::State::log) {
	    $terminal->print(ircformat('logstop0'));
	    $terminal->print(ircformat('logstop1', $NetHirc::State::logname));
	    $NetHirc::State::log->print(ircformat('logstoptext', $$, scalar(gmtime(time()))));
	    $NetHirc::State::log->close;
	    undef $NetHirc::State::log;
	    undef $NetHirc::State::logname;
	} else {
	    $terminal->print(ircformat('Enotlogging'));
	}
    }
}

# bigbrother on
# bigbrother off
# bigbrother set (Pa|Pk|Pr|limit)
# bigbrother verboten

sub term_bigbrother {
    my ($terminal, $event) = @_;
    my $bb = $terminal->param('bigbrother');
    my $rest = $event->args();
    my $first = $event->type();
    my $conn = NetHirc::Server->current()->conn();
    if (ref $bb ne 'NetHirc::BigBrother') {
	$terminal->init_bigbrother();
	$bb = $terminal->param('bigbrother');
	$bb->{'toggle'} = 0;
    }
    if ($rest =~ /^\s*$/) {
	$terminal->print(ircformat('bigbrotherstat', &print_boolean($bb->{'toggle'}, "disabled", "enabled")));
	if ($bb->{'toggle'}) {
	    &term_verboten($terminal, $event);
	    $terminal->print(ircformat('bigbrotherp', $bb->{'Pr'}, $bb->{'Pk'}, $bb->{'Pd'}, $bb->{'Pa'}));
	    $terminal->print(ircformat('bigbrotherl', $bb->{'limit'}));
	}
	return;
    }
    if ($rest =~ /^\s*set\s+(P[akrd])\s+(\d+)/) {
	if ($2 != 0) {
	    $bb->{$1} = $2;
	}
	$terminal->print(ircformat('bigbrotherp', $bb->{'Pr'}, $bb->{'Pk'}, $bb->{'Pd'}, $bb->{'Pa'}));
	return;
    }
    if ($rest =~ /^\s*set\s+limit\s+(\d+)/) {
	my $limit = $1;
	if ($limit != 0) {
	    $bb->{'limit'} = $limit;
	}
	$terminal->print(ircformat('bigbrotherl', $bb->{'limit'}));
	return;
    }
    if ($rest =~ /^\s*verboten/) {
	term_verboten($terminal, $event);
	return;
    }
    if ($rest =~ /^\s*(on|off)/) {
	my $b = &get_boolean($1);
	$bb->{'toggle'} = $b;
	if ($b) {
	    $terminal->print(ircformat('bigbrother_on0'));
	    $terminal->print(ircformat('bigbrother_on1'));
	} else {
	    $terminal->print(ircformat('bigbrother_off0'));
	    $terminal->print(ircformat('bigbrother_off1'));
	}
	return;
    }
    $terminal->print(ircformat('bigbrothersyn'));
}

sub term_verboten {
    my ($terminal, $event) = @_;
    my $bb = $terminal->param('bigbrother');
    return unless $bb;
    return unless $bb->{'toggle'};
    my @verboten = $bb->verboten();
    if (@verboten) {
	$terminal->print(ircformat('verboten4'));
	$terminal->print(ircformat('verboten1', join(' ', map { "'$_'" } @verboten)));
    } else {
	$terminal->print(ircformat('verboten5'));
    }
}

sub term_my {
    my ($terminal, $event) = @_;
    my $conn = NetHirc::Server->current()->conn();
    my $rest = $event->args();
    &term_me($terminal, new NetHirc::TerminalEvent(Type => "me", Args => "'s $rest"));
}

sub term_next {
    my ($terminal, $event) = @_;
    my @channels = NetHirc::Channel->list();
    if (@channels == 0) {
	$terminal->print(ircformat('nochanlist'));
	return;
    }
    if (@channels == 1) {
	$terminal->print(ircformat('onechannel', $channels[0]->name()));
	return;
    }
    fake_join_or_switch(NetHirc::Channel->next());
    update_statusline(0);
}

sub term_set {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    if (not $rest) {
	NetHirc::Helpers::dump_settings();
    } else {
	my ($var, $val) = split(' ', $rest, 2);
	eval "\$NetHirc::State::$var = $val;";
    }
}

# count (status)
# count show (this server only)
# count show <server>
# count show all (all servers)
# count file <file>
# count on
# count off
# count clear (this server only)
# count clear <server>
# count clear all (all servers)

sub term_count {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my $countdb = $NetHirc::State::countdb;
    if ($rest =~ /^\s*$/) {
	if (ref $countdb ne 'NetHirc::StatsDB') {
	    $terminal->print(ircformat('nocount'));
	    return;
	}
	$terminal->print(ircformat('countstate', $countdb->dbpath(), print_boolean($NetHirc::State::counting, "off", "on")));
	return;
    }
    if ($rest =~ /^\s*show\s+(\S+)/) {
	if ($1 eq 'all') {
	    print_counts_all();
	} else {
	    print_counts_server($1);
	}
	return;
    }
    if ($rest =~ /^\s*show$/) {
	print_counts_server(NetHirc::Server->current()->name());
	return;
    }
    if ($rest =~ /^\s*file\s+(\S+)/) {
	my $dbname = $1;
	if (ref $countdb eq 'NetHirc::StatsDB') {
	    $countdb->close();
	    $terminal->print(ircformat('countclose', $countdb->dbpath()));
	}
	$NetHirc::State::countdbname = $dbname;
	$countdb = new NetHirc::StatsDB($dbname);
	$NetHirc::State::countdb = $countdb;
	$terminal->print(ircformat('countopen', $countdb->dbpath()));
	$terminal->print(ircformat('countstate', $countdb->dbpath(), print_boolean($NetHirc::State::counting, "off", "on")));
	return;
    }
    if ($rest =~ /^\s*clear\s+(\S+)/) {
	if ($1 eq 'all') {
	    $countdb->clear_all();
	} else {
	    $countdb->clear_server($1);
	}
	return;
    }
    if ($rest =~ /^\s*(on|off)/) {
	my $bool = $1;
	if (ref $countdb ne 'NetHirc::StatsDB') {
	    $countdb = new NetHirc::StatsDB($NetHirc::State::countdbname);
	    $NetHirc::State::countdb = $countdb;
	    $terminal->print(ircformat('countopen', $countdb->dbpath()));
	    $NetHirc::State::countdbname = $countdb->dbpath();
	}
	$NetHirc::State::counting = get_boolean($bool);
	$terminal->print(ircformat('countstate', $NetHirc::State::countdbname, print_boolean($bool, "off", "on")));
	return;
    }
}

sub term_disconnect {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    my $server = NetHirc::Server->current();
    $server->conn()->quit($rest);
}

sub term_source {
    my ($terminal, $event) = @_;
    my $filename = $event->args();
    $terminal->print(ircformat('sourcestart', $filename));
    my $fh = new IO::File($filename, "r");
    my $line;
    if (not $fh) {
	$terminal->print(ircformat('Esource', $!));
	return;
    }
    while ($line = $fh->getline()) {
	$terminal->process($line);
    }
    $terminal->print(ircformat('sourcedone', $filename));
}

sub term_slashhappy {
    my ($terminal, $event) = @_;
    $terminal->print(ircformat('Whappyslash'));
}

sub term_error {
    my ($terminal, $event) = @_;
    my $rest = $event->args();
    $terminal->print(ircformat('Enocmd', $rest));
}

sub term_empty {
    my ($terminal, $event) = @_;
    $terminal->complain();
}

################################
##### Other Random Helpers #####
################################

sub update_statusline {
    return if $NetHirc::State::first_connect == 1;
    my $terminal = $NetHirc::State::terminal;
    $terminal->display_status_line();
    if ($_[0]) {
	my $srv = NetHirc::Server->current();
	my $now = time();
	my $when = 60 - ($now % 60);
	if ($srv) {
	    my $conn = $srv->conn();
	    $conn->schedule($when, sub { update_statusline(1); } );
	} else {
	    $SIG{'ALRM'} = sub { update_statusline(1); };
	    alarm($when);
	}
    }
}

sub log_n_print {
    my $conn = shift;
    my $format = shift;
    my $terminal = $NetHirc::State::terminal;
    my $text = ircformat($format, @_);
    $terminal->print($text);
    logprint($conn, $text);
}

sub logprint {
    my $conn = shift;
    my $text = shift;
    my $log = $NetHirc::State::log;
    return unless $log;
    my $now = time();
    $log->printf("%s %s %s", $now, $conn->server(), $text);
}

sub server_print {
    my $conn = shift;
    my $text = shift;
    my $terminal = $NetHirc::State::terminal;
    log_n_print($conn, 'server', $text);
}

sub fake_join_or_switch {
    my $chobj = shift;
    my $term = $NetHirc::State::terminal;
    my $srvname = $chobj->conn()->server();
    NetHirc::Server->set($srvname);
    my $ev = new NetHirc::TerminalEvent(Type => 'join', Args => $chobj->name());
    $ev->handler($term);
}

sub totalitarian_check {
    my $text = shift;
    my $bb = $NetHirc::State::terminal->param('bigbrother');
    return unless $bb;
    $bb->check($text);
}

sub timeformat {
    my $now = shift;
    my $str = POSIX::strftime($irc_formats{'time'}, localtime($now));
    chomp $str;
    return $str;
}

sub get_current_conn {
    my $server = NetHirc::Server->current();
    return undef unless $server;
    return $server->conn();
}

sub select_weapon {
    my $hoser = shift;
    unless ($NetHirc::State::weapons{$hoser}) {
	$NetHirc::State::weapons{$hoser} = NetHirc::Helpers::pickrandom(@weapons);
    }
    return $NetHirc::State::weapons{$hoser};
}

sub get_boolean {
    my $text = shift;
    if ($text =~ /(?:y|on|1|yes|true)/i) {
	return 1;
    }
    return 0;
}

sub print_boolean {
    my $bool = shift;
    my $zero = shift;
    my $one = shift;
    return $one if $bool;
    return $zero;
}

sub print_counts_server {
    my $terminal = $NetHirc::State::terminal;
    my $countdb = $NetHirc::State::countdb;
    my $srvname = shift;
    my %stats;
    if (ref $countdb ne 'NetHirc::StatsDB') {
	$terminal->print(ircformat('nocount'));
	return;
    }
    for my $k ($countdb->keys()) {
	next unless ($k =~ /^$srvname/i);
	my ($server, $user, $type) = split(/:/, $k);
	$stats{$user}->[$type] = $countdb->value($k);
    }
    $terminal->print(ircformat('countheader', $srvname));
    for my $user (sort { $stats{$a}->[0] <=> $stats{$b}->[0] } keys %stats) {
	$terminal->print(ircformat('count', $user, @{$stats{$user}}));
    }
    $terminal->print(ircformat('countfooter', $srvname));
}

sub print_counts_all {
    my $terminal = $NetHirc::State::terminal;
    my $countdb = $NetHirc::State::countdb;
    if (ref $countdb ne 'NetHirc::StatsDB') {
	$terminal->print(ircformat('nocount'));
	return;
    }
    for my $s (NetHirc::Server->list()) {
	print_counts_server($s->name());
    }
}

sub update_counters {
    return unless $NetHirc::State::countdb;
    return unless $NetHirc::State::counting;
    my $terminal = $NetHirc::State::terminal;
    my $countdb = $NetHirc::State::countdb;
    my $conn = shift;
    my $event = shift;
    $countdb->add_event($conn, $event);
}

sub reconnect_old_channels {
    my $conn = shift;
    my @reconn = NetHirc::Channel->needs_reconnect($conn);
    for my $c (@reconn) {
	my $ev = new NetHirc::TerminalEvent(Type => 'join', Args => $c->name());
	$ev->handler($NetHirc::State::terminal);
    }
}

sub join_commandline_channels {
    my $conn = shift;
    if ($NetHirc::State::options{'j'}) {
	my @channels = split(/,/, $NetHirc::State::options{'j'});
	for my $c (@channels) {
	    my $ev = new NetHirc::TerminalEvent(Type => 'join', Args => $c);
	    $ev->handler($NetHirc::State::terminal);
	}
	delete $NetHirc::State::options{'j'};
    }
}

1;
