package NetHirc::Helpers;

use NetHirc::Channel;
use NetHirc::Formats;

sub yoda_front {
    my @a = split(/([-`!&(){}\[\]<>":;,.?=\\|^\$#_@]+)/, $_[0]);
    my @w;
    my $b;
    for my $segment (@a) {
        @w = split(/(\s+)/, $segment);
        $b .= &yoda(@w);
    }
    return $b;
}

sub yoda {
    my (@words) = @_;
    my ($x, $y);
    my ($iterations);
    $iterations = $#words;
    if (join("", @words) !~ /\S/) {
        return join "", @words;
    }
    for (1..$iterations) {
        while (1) {
            $x = rand($#words + 1);
            last if ($words[$x] =~ /\S/);
        }
        while (1) {
            $y = rand($#words + 1);
            last if ($words[$y] =~ /\S/);
        }
        ($words[$x], $words[$y]) = +($words[$y], $words[$x])
    }
    return join "", @words;
}

sub pigl_front {
    my @a = split(/(\b)/, $_[0]);
    my @b = ();
    for my $i (@a) {
	push @b, &pigl($i);
    }
    return join "", @b;
}

sub pigl {
    my $garply = shift;
    return $garply if ($garply =~ /^(a(nd?|[st])?|t(o|he)|o[frn]|i[fnst]?)$/oi);
    return $garply if ($garply =~ /^(\W|\d|\s)*$/);
    $garply =~ s/^([^a-zA-Z]*)([^aeiouyAEIOUY]*)([a-zA-Z']*)(.*)/$1$3$2ay$4/;
    return $garply;
}

sub mirror {
    my $line = shift;
    my $enil;
    chomp($line);
    while ($line =~ /\t/) {	# untab for effective mirroring.
	my $spaces = " " x (8 - (index($line, "\t") % 8));
	$line =~ s/\t/$spaces/;
    }
    $enil = scalar reverse $line;
    $enil =~ tr,\\/\(\)\[\]{}<>`',/\\\)\(\]\[}{><'`,;
    return $enil;
}

sub pickrandom { $_[rand @_] }

sub modesplitter {
    my $r_modelist = shift;
    my $modestring = shift;
    my @targets = @_;
    my ($parity, $modechar);
    ## Why this way?
    $modestring = reverse $modestring;
    ## Because it's silly and wrong.  Besides, splitting on // is dull. :-)
    while ($modechar = chop($modestring)) {
	if (($modechar eq '+') || ($modechar eq '-')) {
	    $parity = $modechar;
	} else {
	    if ($modechar =~ /[bov]/) {
		push(@$r_modelist, [ $parity . $modechar, shift(@targets) ]);
	    } elsif ($modechar =~ /[kl]/) {
		if ($parity eq '+') {
		    push(@$r_modelist, [ $parity . $modechar, shift(@targets) ]);
		} else {
		    push(@$r_modelist, [ $parity . $modechar ]);
		}
	    } else {
		push(@$r_modelist, [ $parity . $modechar ]);
	    }
	}
    }
}

sub saytoer {
## It's another one of these "Joy of Lambda" subs.
    my ($conn, $destination, $fh, $filename) = @_;
    if (ref $destination eq 'NetHirc::Channel') {
	$destination = $destination->name();
    }
    return sub {
    ## Takes the given text and writes it to the destination 
    ## (person or channel) with Net::IRC::Connection::privmsg().
	my $line;
	my $terminal = $NetHirc::State::terminal;
	if ($fh->eof()) {
	    $NetHirc::State::irc->removefh($fh);
	    $terminal->print(&ircformat('pipeclosed', $filename, $destination));
	    $fh->close();
	    return;
	}
	$line = $fh->getline;
	chomp $line;
	my $ev;
	if (NetHirc::Channel->is_channel_name($destination)) {
	    $ev = new NetHirc::TerminalEvent(Type => 'say', Args => $line);
	} else {
	    $ev = new NetHirc::TerminalEvent(Type => 'msg', Args => "$destination $line");
	}
	$ev->handler($terminal);
	sleep(1);
    };
}

sub admin_print {
    my $terminal = $NetHirc::State::terminal;
    $terminal->print(ircformat('admin', @_));
}

1;
