package NetHirc::Server;

use strict;

use Carp;

my @servers = ();
my %conns = ();

sub new {
    my $proto = shift;
    my $class = ref $proto || $proto;
    my $conn = shift;
    if (ref $conn ne 'Net::IRC::Connection') {
	croak __PACKAGE__, "::new -- must provide a Net::IRC::Connection reference";
    }
    my $self = $conn->server();
    $conns{$self} = $conn;
    return bless \$self, $class;
}

sub add {
    my $s = new(@_);
    push @servers, $s;
}

sub set { 
    shift;
    my $server = shift;
    my $item;
    my @net = grep { $$_ eq $server } @servers;
    if (not @net) {
	return undef;
    }
    $item = $net[0];
    rotate_to_front($item);
    return $item;
}

sub remove {
    shift;
    my $server = shift;
    @servers = grep { $$_ ne $server } @servers;
    delete $conns{$server};
}

sub current { shift; $servers[0] }

sub rotate_to_front {
    my $item = shift;
    while ($servers[0] ne $item) {
	push @servers, shift @servers;
    }
}

sub list { shift; @servers }

sub conn { 
    my $self = shift;
    my $name = $$self;
    return $conns{$name};
}

sub name { 
    my $self = shift;
    return $$self;
}

sub debug {
    shift;
    my $bool = shift;
    map { $_->debug($bool) } values %conns;
}

1;
