# (c) 2000-2002 Tony Monroe

package NetHirc::StatsDB;

use strict;

BEGIN { @AnyDBM_File::ISA = qw(DB_File GDBM_File NDBM_File SDBM_File); }
use AnyDBM_File;

use Net::IRC::Connection;
use Net::IRC::Event;

use constant SDB_DBH => 0;
use constant SDB_HASHREF => 1;
use constant SDB_PATH => 2;

my $rcs_id = q$Id: StatsDB.pm,v 1.2 2002/04/28 01:32:03 tony Exp $;

# server:sender:0	=>	number of lines
# server:sender:1	=>	number or words
# server:sender:2	=>	number of bytes

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $path = shift;
    my %db = ();
    my $dbh = tie %db, 'DB_File', $path;
    return undef unless $dbh;
    my $self = [ $dbh, \%db, $path ];
    return bless $self, $class;
}

sub add_event {
    my $self = shift;
    my $conn = shift;
    my $event = shift;
    my $sender = $event->nick();
    my ($text) = $event->args();
    my $bytes = length $text;
    my @words = split(' ', $text);
    $self->add_bytes($conn, $sender, $bytes);
    $self->add_words($conn, $sender, scalar @words);
    $self->add_line($conn, $sender);
}

sub add_bytes {
    my $self = shift;
    my $conn = shift;
    my $sender = shift;
    my $bytes = shift;
    my $k = sprintf("%s:%s:2", $conn->server(), $sender);
    $self->add($k, $bytes);
}

sub add_line {
    my $self = shift;
    my $conn = shift;
    my $sender = shift;
    my $k = sprintf("%s:%s:0", $conn->server(), $sender);
    $self->add($k, 1);
}

sub add_words {
    my $self = shift;
    my $conn = shift;
    my $sender = shift;
    my $words = shift;
    my $k = sprintf("%s:%s:1", $conn->server(), $sender);
    $self->add($k, $words);
}

sub add {
    my $self = shift;
    my $k = shift;
    my $num = shift;
    $self->[SDB_DBH]->sync();
    $self->[SDB_HASHREF]->{$k} += $num;
    $self->[SDB_DBH]->sync();
}

sub keys {
    my $self = shift;
    keys %{$self->[SDB_HASHREF]};
}

sub value {
    my $self = shift;
    my $k = shift;
    $self->[SDB_HASHREF]->{$k};
}

sub dbpath {
    my $self = shift;
    $self->[SDB_PATH];
}

sub delete {
    my $self = shift;
    my $key = shift;
    delete $self->[SDB_HASHREF]->{$key};
}

sub close {
    my $self = shift;
    $self->[SDB_DBH]->sync();
    untie $self->[SDB_DBH];
    undef $self->[SDB_HASHREF];
}

sub clear_all {
    my $self = shift;
    my $conn = shift;
    for my $k ($self->keys()) {
	$self->delete($k);
    }
}

sub clear_server {
    my $self = shift;
    my $srvname = shift;
    for my $k ($self->keys()) {
	if ($k =~ /$srvname/) {
	    $self->delete($k);
	}
    }
}

1;
__END__

