#!/usr/local/bin/perl

# (c) 2002 Tony Monroe

my $rcs_id = q$Id: nethirc.pl,v 1.11 2004/05/04 21:01:00 tony Exp $;

use strict;
use vars qw( $irc $terminal );

use Carp;
use File::Basename;
use Getopt::Std;
use Net::IRC;

use NetHirc;
use NetHirc::Formats;
use NetHirc::Handlers;
use NetHirc::Server;
use NetHirc::Terminal;
use NetHirc::TerminalEvent;

my $progname = basename($0);
my @personal_info = getpwuid($<);
my $login = $personal_info[0];
my $gecos = $personal_info[6];
my $homedir = $personal_info[7];

my %opts = (
    'c'	=>	"complaints.txt",
    'f' =>	"$homedir/.nethirc",
    'i'	=>	$ENV{'IRCNAME'} || $gecos,
    'n'	=>	$ENV{'IRCNICK'} || $login,
    'p'	=>	6667,
    'u'	=>	$login,
    't' => 	"$homedir/.nethirc.pl",
);

getopts('c:f:i:j:n:p:s:t:u:d?', \%opts);

&display_usage if $opts{'?'};

my $rcfile = $opts{'f'};
my $ircname = $opts{'i'};
my $nick = $opts{'n'};
my $port = $opts{'p'};
my $server = $opts{'s'};
my $username = $opts{'u'};
my $complaints = $opts{'c'};
my $debug = $opts{'d'};
my $tweakfile = $opts{'t'};

&display_usage() unless $opts{'s'};

my $irc = new Net::IRC;
my $terminal = new NetHirc::Terminal(
	\*STDIN, 
	Nickname => $nick, 
	Complaints => $complaints,
	Output => \*STDOUT,
);

croak "Can't set up terminal: $!" unless $terminal;

if ($@) {
    die "Can't set up initial IRC connection: $!";
}

my $conn = $irc->newconn( Nick => $opts{'n'},
			   Ircname => $opts{'i'},
			   Username => $opts{'u'},
			   Server => $opts{'s'},
			   Port => $opts{'p'},
);
$conn->debug($debug);

NetHirc::Server->add($conn);
NetHirc::Server->set($conn->server);

$NetHirc::State::irc = $irc;
$NetHirc::State::terminal = $terminal;
%NetHirc::State::options = %opts;

&init_chat_handlers();
&init_term_handlers();
&read_rcfile($rcfile);
&read_customizations($tweakfile);

$irc->addfh($terminal, \&NetHirc::Terminal::spit, "r");
$irc->timeout(10);
eval { $irc->start(); };
print $@ unless ($@ =~ /^term_quit/);
&cleanup();
exit 0;

sub init_chat_handlers {
    $conn->add_global_handler('alreadyregistered', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('away', \&NetHirc::Handlers::on_away);
    $conn->add_global_handler('badchanmask', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('badchannelkey', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('banlist', \&NetHirc::Handlers::on_banlist);
    $conn->add_global_handler('bannedfromchan', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('caction', \&NetHirc::Handlers::on_caction);
    $conn->add_global_handler('cannotsendtochan', \&NetHirc::Handlers::on_cannotsendtochan);
    $conn->add_global_handler('cantkillserver', \&NetHirc::Handlers::on_cantkillserver);
    $conn->add_global_handler('cdcc', \&NetHirc::Handlers::on_ignore);
    $conn->add_global_handler('cfinger', \&NetHirc::Handlers::on_cfinger);
    $conn->add_global_handler('channelcreate', \&NetHirc::Handlers::on_channelcreate);
    $conn->add_global_handler('channelisfull', \&NetHirc::Handlers::on_server);
    $conn->add_global_handler('channelmodeis', \&NetHirc::Handlers::on_channelmodeis);
    $conn->add_global_handler('chanoprivsneeded', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('chat', \&NetHirc::Handlers::on_chat);
    $conn->add_global_handler('clientinfo', \&NetHirc::Handlers::on_clientinfo);
    $conn->add_global_handler('closeend', \&NetHirc::Handlers::on_init1);
    $conn->add_global_handler('closing', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('connect', \&NetHirc::Handlers::on_connect);
    $conn->add_global_handler('cping', \&NetHirc::Handlers::on_cping);
    $conn->add_global_handler('crclientinfo', \&NetHirc::Handlers::on_crclientinfo);
    $conn->add_global_handler('created', \&NetHirc::Handlers::on_init2);
    $conn->add_global_handler('crfinger', \&NetHirc::Handlers::on_crfinger);
    $conn->add_global_handler('crping', \&NetHirc::Handlers::on_crping);
    $conn->add_global_handler('crsource', \&NetHirc::Handlers::on_crsource);
    $conn->add_global_handler('crtime', \&NetHirc::Handlers::on_crtime);
    $conn->add_global_handler('cruserinfo', \&NetHirc::Handlers::on_cruserinfo);
    $conn->add_global_handler('crversion', \&NetHirc::Handlers::on_crversion);
    $conn->add_global_handler('csource', \&NetHirc::Handlers::on_csource);
    $conn->add_global_handler('ctime', \&NetHirc::Handlers::on_ctime);
    $conn->add_global_handler('cuserinfo', \&NetHirc::Handlers::on_cuserinfo);
    $conn->add_global_handler('cversion', \&NetHirc::Handlers::on_cversion);
    $conn->add_global_handler('dcc_close', \&NetHirc::Handlers::on_dcc_close);
    $conn->add_global_handler('dcc_open', \&NetHirc::Handlers::on_dcc_open);
    $conn->add_global_handler('dcc_update', \&NetHirc::Handlers::on_dcc_update);
    $conn->add_global_handler('disconnect', \&NetHirc::Handlers::on_disconnect);
    $conn->add_global_handler('endofbanlist', \&NetHirc::Handlers::on_ignore);
    $conn->add_global_handler('endofinfo', \&NetHirc::Handlers::on_ignore);
    $conn->add_global_handler('endoflinks', \&NetHirc::Handlers::on_ignore);
    $conn->add_global_handler('endofmotd', \&NetHirc::Handlers::on_endofmotd);
    $conn->add_global_handler('endofnames', \&NetHirc::Handlers::on_ignore);
    $conn->add_global_handler('endofservices', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('endoftrace', \&NetHirc::Handlers::on_ignore);
    $conn->add_global_handler('endofusers', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('endoftrace', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('endofwho', \&NetHirc::Handlers::on_ignore);
    $conn->add_global_handler('endofwhois', \&NetHirc::Handlers::on_ignore);
    $conn->add_global_handler('endofwhowas', \&NetHirc::Handlers::on_ignore);
    $conn->add_global_handler('erroneusnickname', \&NetHirc::Handlers::on_nickgoof);
    $conn->add_global_handler('fileerror', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('info', \&NetHirc::Handlers::on_init1);
    $conn->add_global_handler('infostart', \&NetHirc::Handlers::on_init2);
    $conn->add_global_handler('invite', \&NetHirc::Handlers::on_invite);
    $conn->add_global_handler('inviteonlychan', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('inviting', \&NetHirc::Handlers::on_inviting);
    $conn->add_global_handler('ison', \&NetHirc::Handlers::on_ison);
    $conn->add_global_handler('join', \&NetHirc::Handlers::on_join);
    $conn->add_global_handler('keyset', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('kick', \&NetHirc::Handlers::on_kick);
    $conn->add_global_handler('kill', \&NetHirc::Handlers::on_kill);
    $conn->add_global_handler('killdone', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('leaving', \&NetHirc::Handlers::on_leaving);
    $conn->add_global_handler('links', \&NetHirc::Handlers::on_links);
    $conn->add_global_handler('list', \&NetHirc::Handlers::on_list);
    $conn->add_global_handler('listend', \&NetHirc::Handlers::on_listend);
    $conn->add_global_handler('liststart', \&NetHirc::Handlers::on_liststart);
    $conn->add_global_handler('luserchannels', \&NetHirc::Handlers::on_init1);
    $conn->add_global_handler('luserclient', \&NetHirc::Handlers::on_init2);
    $conn->add_global_handler('luserconns', \&NetHirc::Handlers::on_init2);
    $conn->add_global_handler('luserme', \&NetHirc::Handlers::on_init2);
    $conn->add_global_handler('luserop', \&NetHirc::Handlers::on_init1);
    $conn->add_global_handler('luserunknown', \&NetHirc::Handlers::on_init1);
    $conn->add_global_handler('mode', \&NetHirc::Handlers::on_mode);
    $conn->add_global_handler('motd', \&NetHirc::Handlers::on_motd);
    $conn->add_global_handler('motd2', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('motdstart', \&NetHirc::Handlers::on_init2);
    $conn->add_global_handler('msg', \&NetHirc::Handlers::on_msg);
    $conn->add_global_handler('myinfo', \&NetHirc::Handlers::on_init1);
    $conn->add_global_handler('myportis', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('n_global', \&NetHirc::Handlers::on_init2);
    $conn->add_global_handler('n_local', \&NetHirc::Handlers::on_init2);
    $conn->add_global_handler('namreply', \&NetHirc::Handlers::on_namreply);
    $conn->add_global_handler('needmoreparams', \&NetHirc::Handlers::on_needmoreparams);
    $conn->add_global_handler('nick', \&NetHirc::Handlers::on_nick);
    $conn->add_global_handler('nickcollision', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('nicknameinuse', \&NetHirc::Handlers::on_nicknameinuse);
    $conn->add_global_handler('noadmininfo', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('noaway', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('nologin', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('nomotd', \&NetHirc::Handlers::on_nomotd);
    $conn->add_global_handler('none', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('nonicknamegiven', \&NetHirc::Handlers::on_nickgoof);
    $conn->add_global_handler('nooperhost', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('noorigin', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('nopermforhost', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('noprivileges', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('norecipient', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('noservicehost', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('nosuchchannel', \&NetHirc::Handlers::on_nosuchchannel);
    $conn->add_global_handler('nosuchnick', \&NetHirc::Handlers::on_nosuchnick);
    $conn->add_global_handler('nosuchserver', \&NetHirc::Handlers::on_nosuchserver);
    $conn->add_global_handler('notexttosend', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('notice', \&NetHirc::Handlers::on_notice);
    $conn->add_global_handler('notonchannel', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('notopic', \&NetHirc::Handlers::on_notopic);
    $conn->add_global_handler('notoplevel', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('notregistered', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('nousers', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('nowaway', \&NetHirc::Handlers::on_switchaway);
    $conn->add_global_handler('other', \&NetHirc::Handlers::on_other);
    $conn->add_global_handler('part', \&NetHirc::Handlers::on_part);
    $conn->add_global_handler('passwdmismatch', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('public', \&NetHirc::Handlers::on_public);
    $conn->add_global_handler('quit', \&NetHirc::Handlers::on_quit);
    $conn->add_global_handler('rehashing', \&NetHirc::Handlers::on_init1);
    $conn->add_global_handler('service', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('serviceinfo', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('servlist', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('servlistend', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('snotice', \&NetHirc::Handlers::on_notice);
    $conn->add_global_handler('statscline', \&NetHirc::Handlers::on_stats);
    $conn->add_global_handler('statscommands', \&NetHirc::Handlers::on_stats);
    $conn->add_global_handler('statshline', \&NetHirc::Handlers::on_stats);
    $conn->add_global_handler('statsiline', \&NetHirc::Handlers::on_stats);
    $conn->add_global_handler('statskline', \&NetHirc::Handlers::on_stats);
    $conn->add_global_handler('statslinkinfo', \&NetHirc::Handlers::on_stats);
    $conn->add_global_handler('statslline', \&NetHirc::Handlers::on_stats);
    $conn->add_global_handler('statsnline', \&NetHirc::Handlers::on_stats);
    $conn->add_global_handler('statsoline', \&NetHirc::Handlers::on_stats);
    $conn->add_global_handler('statsqline', \&NetHirc::Handlers::on_stats);
    $conn->add_global_handler('statsuptime', \&NetHirc::Handlers::on_stats);
    $conn->add_global_handler('statsyline', \&NetHirc::Handlers::on_stats);
    $conn->add_global_handler('summondisabled', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('summoning', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('time', \&NetHirc::Handlers::on_time);
    $conn->add_global_handler('toomanychannels', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('toomanytargets', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('topic', \&NetHirc::Handlers::on_topic);
    $conn->add_global_handler('topicinfo', \&NetHirc::Handlers::on_topicinfo);
    $conn->add_global_handler('traceclass', \&NetHirc::Handlers::on_traceclass);
    $conn->add_global_handler('traceclass', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('traceconnecting', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('tracehandshake', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('tracelink', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('tracelog', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('tracenewtype', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('traceoper', \&NetHirc::Handlers::on_traceoper);
    $conn->add_global_handler('traceoperator', \&NetHirc::Handlers::on_init1);
    $conn->add_global_handler('traceserver', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('traceunknown', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('traceuser', \&NetHirc::Handlers::on_init1);
    $conn->add_global_handler('umodeis', \&NetHirc::Handlers::on_umodeis);
    $conn->add_global_handler('umodeunknownflag', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('unaway', \&NetHirc::Handlers::on_switchaway);
    $conn->add_global_handler('unknowncommand', \&NetHirc::Handlers::on_unknowncommand);
    $conn->add_global_handler('unknownmode', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('userhost', \&NetHirc::Handlers::on_userhost);
    $conn->add_global_handler('usernotinchannel', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('useronchannel', \&NetHirc::Handlers::on_useronchannel);
    $conn->add_global_handler('users', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('usersdisabled', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('usersdontmatch', \&NetHirc::Handlers::on_goof);
    $conn->add_global_handler('usersstart', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('version', \&NetHirc::Handlers::on_init1);
    $conn->add_global_handler('version', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('wallops', \&NetHirc::Handlers::on_wallops);
    $conn->add_global_handler('wasnosuchnick', \&NetHirc::Handlers::on_wasnosuchnick);
    $conn->add_global_handler('welcome', \&NetHirc::Handlers::on_init2);
    $conn->add_global_handler('whoischannels', \&NetHirc::Handlers::on_whoischannels);
    $conn->add_global_handler('whoischanop', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('whoisidle', \&NetHirc::Handlers::on_whoisidle);
    $conn->add_global_handler('whoisoperator', \&NetHirc::Handlers::on_whoisoperator);
    $conn->add_global_handler('whoisserver', \&NetHirc::Handlers::on_whoisserver);
    $conn->add_global_handler('whoisuser', \&NetHirc::Handlers::on_whoisuser);
    $conn->add_global_handler('whoreply', \&NetHirc::Handlers::on_whoreply);
    $conn->add_global_handler('whowasuser', \&NetHirc::Handlers::on_whowasuser);
    $conn->add_global_handler('wildtoplevel', \&NetHirc::Handlers::on_unknown);
    $conn->add_global_handler('yourebannedcreep', \&NetHirc::Handlers::on_yourebannedcreep);
    $conn->add_global_handler('youreoper', \&NetHirc::Handlers::on_youreoper);
    $conn->add_global_handler('yourhost', \&NetHirc::Handlers::on_init2);
    $conn->add_global_handler('youwillbebanned', \&NetHirc::Handlers::on_unknown);

}


# Here is how we define the command set.
sub init_term_handlers {
    $terminal->add_handler('!!', \&NetHirc::Handlers::term_pipe);
    $terminal->add_handler('!', \&NetHirc::Handlers::term_system);
    $terminal->add_handler('!m', \&NetHirc::Handlers::term_msgpipe);
    $terminal->add_handler('.', \&NetHirc::Handlers::term_source);
    $terminal->add_handler('?', \&NetHirc::Handlers::term_help);
    $terminal->add_handler('action', \&NetHirc::Handlers::term_me);
    $terminal->add_handler('admin', \&NetHirc::Handlers::term_admin);
    $terminal->add_handler('away', \&NetHirc::Handlers::term_away);
    $terminal->add_handler('back', \&NetHirc::Handlers::term_back);
    $terminal->add_handler('ban', \&NetHirc::Handlers::term_ban);
    $terminal->add_handler('bigbrother', \&NetHirc::Handlers::term_bigbrother);
    $terminal->add_handler('bye', \&NetHirc::Handlers::term_quit);
    $terminal->add_handler('cc', \&NetHirc::Handlers::term_cmdchar);
    $terminal->add_handler('channel', \&NetHirc::Handlers::term_join_or_switch);
    $terminal->add_handler('channels', \&NetHirc::Handlers::term_channels);
    $terminal->add_handler('cmdchar', \&NetHirc::Handlers::term_cmdchar);
    $terminal->add_handler('count', \&NetHirc::Handlers::term_count);
    $terminal->add_handler('ctcp', \&NetHirc::Handlers::term_ctcp);
    $terminal->add_handler('dc', \&NetHirc::Handlers::term_disconnect);
    $terminal->add_handler('debug', \&NetHirc::Handlers::term_debug);
    $terminal->add_handler('disconnect', \&NetHirc::Handlers::term_disconnect);
    $terminal->add_handler('echo', \&NetHirc::Handlers::term_echo);
    $terminal->add_handler('empty', \&NetHirc::Handlers::term_empty);
    $terminal->add_handler('error', \&NetHirc::Handlers::term_error);
    $terminal->add_handler('eval', \&NetHirc::Handlers::term_eval);
    $terminal->add_handler('happyslash', \&NetHirc::Handlers::term_happyslash);
    $terminal->add_handler('help', \&NetHirc::Handlers::term_help);
    $terminal->add_handler('ignore', \&NetHirc::Handlers::term_ignore);
    $terminal->add_handler('info', \&NetHirc::Handlers::term_info);
    $terminal->add_handler('inventory', \&NetHirc::Handlers::term_channels);
    $terminal->add_handler('invite', \&NetHirc::Handlers::term_invite);
    $terminal->add_handler('ison', \&NetHirc::Handlers::term_ison);
    $terminal->add_handler('join', \&NetHirc::Handlers::term_join_or_switch);
    $terminal->add_handler('key	', \&NetHirc::Handlers::term_key);
    $terminal->add_handler('kick', \&NetHirc::Handlers::term_kick);
    $terminal->add_handler('kickban', \&NetHirc::Handlers::term_kickban);
    $terminal->add_handler('leave', \&NetHirc::Handlers::term_leave);
    $terminal->add_handler('links', \&NetHirc::Handlers::term_links);
    $terminal->add_handler('list', \&NetHirc::Handlers::term_list);
    $terminal->add_handler('log', \&NetHirc::Handlers::term_log);
    $terminal->add_handler('lusers', \&NetHirc::Handlers::term_lusers);
    $terminal->add_handler('m', \&NetHirc::Handlers::term_msg);
    $terminal->add_handler('me', \&NetHirc::Handlers::term_me);
    $terminal->add_handler('mirror', \&NetHirc::Handlers::term_mirror);
    $terminal->add_handler('mirrormsg', \&NetHirc::Handlers::term_mirrormsg);
    $terminal->add_handler('mode', \&NetHirc::Handlers::term_mode);
    $terminal->add_handler('motd', \&NetHirc::Handlers::term_motd);
    $terminal->add_handler('msg', \&NetHirc::Handlers::term_msg);
    $terminal->add_handler('msgpipe', \&NetHirc::Handlers::term_msgpipe);
    $terminal->add_handler('my', \&NetHirc::Handlers::term_my);
    $terminal->add_handler('my', \&NetHirc::Handlers::term_my);
    $terminal->add_handler('names', \&NetHirc::Handlers::term_names);
    $terminal->add_handler('next', \&NetHirc::Handlers::term_next);
    $terminal->add_handler('nick', \&NetHirc::Handlers::term_nick);
    $terminal->add_handler('notice', \&NetHirc::Handlers::term_notice);
    $terminal->add_handler('op', \&NetHirc::Handlers::term_op);
    $terminal->add_handler('oper', \&NetHirc::Handlers::term_oper);
    $terminal->add_handler('part', \&NetHirc::Handlers::term_leave);
    $terminal->add_handler('pickle', \&NetHirc::Handlers::term_quit);
    $terminal->add_handler('pigl', \&NetHirc::Handlers::term_pigl);
    $terminal->add_handler('piglmsg', \&NetHirc::Handlers::term_piglmsg);
    $terminal->add_handler('ping', \&NetHirc::Handlers::term_ping);
    $terminal->add_handler('pipe', \&NetHirc::Handlers::term_pipe);
    $terminal->add_handler('q', \&NetHirc::Handlers::term_quit);
    $terminal->add_handler('quit', \&NetHirc::Handlers::term_quit);
    $terminal->add_handler('quote', \&NetHirc::Handlers::term_quote);
    $terminal->add_handler('reload', \&NetHirc::Handlers::term_reload);
    $terminal->add_handler('reverse', \&NetHirc::Handlers::term_reverse);
    $terminal->add_handler('say', \&NetHirc::Handlers::term_say);
    $terminal->add_handler('sconnect', \&NetHirc::Handlers::term_sconnect);
    $terminal->add_handler('select', \&NetHirc::Handlers::term_join_or_switch);
    $terminal->add_handler('server', \&NetHirc::Handlers::term_server);
    $terminal->add_handler('set', \&NetHirc::Handlers::term_set);
    $terminal->add_handler('slashhappy', \&NetHirc::Handlers::term_slashhappy);
    $terminal->add_handler('source', \&NetHirc::Handlers::term_source);
    $terminal->add_handler('squelch', \&NetHirc::Handlers::term_squelch);
    $terminal->add_handler('squit', \&NetHirc::Handlers::term_squit);
    $terminal->add_handler('stats', \&NetHirc::Handlers::term_stats);
    $terminal->add_handler('system', \&NetHirc::Handlers::term_system);
    $terminal->add_handler('tab', \&NetHirc::Handlers::term_tab);
    $terminal->add_handler('talkto', \&NetHirc::Handlers::term_join_or_switch);
    $terminal->add_handler('time', \&NetHirc::Handlers::term_time);
    $terminal->add_handler('topic', \&NetHirc::Handlers::term_topic);
    $terminal->add_handler('trace', \&NetHirc::Handlers::term_trace);
    $terminal->add_handler('tweak', \&NetHirc::Handlers::term_tweak);
    $terminal->add_handler('unban', \&NetHirc::Handlers::term_unban);
    $terminal->add_handler('userhost', \&NetHirc::Handlers::term_userhost);
    $terminal->add_handler('users', \&NetHirc::Handlers::term_users);
    $terminal->add_handler('verboten', \&NetHirc::Handlers::term_verboten);
    $terminal->add_handler('version', \&NetHirc::Handlers::term_version);
    $terminal->add_handler('wallops', \&NetHirc::Handlers::term_wallops);
    $terminal->add_handler('who', \&NetHirc::Handlers::term_who);
    $terminal->add_handler('who', \&NetHirc::Handlers::term_who);
    $terminal->add_handler('whois', \&NetHirc::Handlers::term_whois);
    $terminal->add_handler('whowas', \&NetHirc::Handlers::term_whowas);
    $terminal->add_handler('yoda', \&NetHirc::Handlers::term_yoda);
    $terminal->add_handler('yodamsg', \&NetHirc::Handlers::term_yodamsg);
}

sub cleanup 
{
}

sub read_rcfile
{
    my $rcfile = shift;
    return unless -e $rcfile;
    my $ev = new NetHirc::TerminalEvent(Type => 'source', Args => $rcfile);
    $ev->handler($terminal);
}

sub read_customizations
{
    my $tweakfile = shift;
    if (-e $tweakfile) 
    { 
	my $ev = new NetHirc::TerminalEvent(Type => 'tweak', Args => $tweakfile);
	$ev->handler($terminal);
    }
}

sub display_usage()
{
    die <<EOF;
Usage: $progname -s server [-f rcfile] [-h localaddr] [-i ircname] [-n nick] [-p port] [-u userfake]
EOF
}
__END__

=pod

=head1 NAME

nethirc - A nethack-flavored IRC client

=head1 SYNOPSIS

B<nethirc>
B<-s> I<server> 
[B<-n> I<nickname>] 
[B<-p> I<port>] 
[B<-i> I<ircname>] 
[B<-h> I<hostname>] 
[B<-c> I<complaints>] 
[B<-t> I<tweaks>]
[B<-j> I<channel>[,I<channel> ...]]

=head1 DESCRIPTION

B<nethirc> is an IRC client written entirely in Perl, with the help of the 
C<Net::IRC>
modules from CPAN.
It is named B<nethirc>
because of the influence of 
B<nethack>,
which is the theme behind many of the things that 
B<nethirc> displays on your screen.

=head1 OPTIONS

B<nethirc>'s command line interface is intentionally a bit different
than that of B<ircII>, arguably the standard IRC client.  The B<-s>
option I<must> be present for things to behave as you expect.  It
selects an IRC I<server> to which to connect.  There is no default.

The B<-n> option is used to specify a I<nickname> to use to connect
to the server you specified with the B<-s> option.  If you do not
explictly specify a nickname, expect to get stuck with the contents
of the IRCNICK environment variable, your login name, or whatever
C<Net::IRC> thinks is appropriate.

The B<-p> option specifies a I<port> (other than the default 6667) to
use to connect to the IRC server.

The B<-i> option specifies a witty string to use as your I<ircname>.
If you don't give one, and the IRCNAME environment variable is not
set, Net::IRC will choose one for you.

The B<-h> option is stolen from ircII, and specifies the I<hostname>
by which your machine is to be known.  If DNS doesn't check out on
this hostname, don't expect the IRC server to like you very much.

The B<-c> option specifies a file full of I<complaints>, one per line,
that B<nethirc> will display if you type blank lines at it.

The B<-j> option specifies a comma-separated list of I<channels> you
want to join immediately after connecting to the named server.

The B<-t> option specifies a file full of I<tweaks> you can load.
Tweaks are Perl code that digs into the guts of the system and 
changes things.  It's not a fancy customization mechanism, but
it works.  Be aware that there is a minimum height requirement.
The default tweakfile is C<~/.nethirc.pl>.

=head1 USAGE

Start the client.  Have fun.  Yell at people.  There is a full
command set at your disposal.  Those used to ircII should have no
problem with the default command set.

=head1 EXIT STATUS

Does it really matter?

=head1 FILES

None yet.

=head1 ENVIRONMENT

The C<IRCNAME> and C<IRCNICK> variables retain their usual meaning.

=head1 SEE ALSO

L<perl(1)>, L<nethack(6)>, L<fortune(6)>.

=head1 BUGS

...are in the eye of the beholder.  If you find one, please let me know.

=head1 AUTHOR

Tony Monroe E<lt>tmonroe plus perl at nog dot netE<gt>, sometimes
known as Eggplant on EFnet.

=head1 HISTORY

B<nethirc> was written in a fit of experimentation and madness and
frustration with a previous creation known as hoserchat.  The main
idea was, of course, "Wouldn't a Nethack-like IRC client be cool?
Or at least amusing?"  And so, several months of on-and-off development
time later, I felt that the world won't wince too much at the sight
of version 0.01 of this program.  So it was released.

It went through a few revisions after that, but it suffered because
its internal architecture was rather stinky.  Starting with version
0.7, its code layout makes much more sense, though it has lost a
good deal of its dynamic-reload capability (which was a gross hack
anyway).  However, it has added some random amusements which I hope
you will find appalling, at the least.
